/*
 * Copyright (c) 2012,2015 Renesas Electronics Corporation and TESSERA
 * TECHNOLOGY INC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//=============================================================================
// CComanndoProcessor NX
//=============================================================================
#include "stdwin.h"
#include "CommandProcessor.h"
# include "Common.h"

//-----------------------------------------------------------------------------
// @outline		RXgN^
// @explanation	RXgN^
// @param		none
// @return		none
//-----------------------------------------------------------------------------
CCommandProcessor::CCommandProcessor(void)
	: m_pbCmdFlames(NULL), m_pbDataFlames(NULL)
{
	// DO NOTHING
}


//-----------------------------------------------------------------------------
// @outline		fXgN^
// @explanation	fXgN^
// @param		none
// @return		none
//-----------------------------------------------------------------------------
CCommandProcessor::~CCommandProcessor(void)
{
    // R}hEt[̔j
	if (NULL != m_pbCmdFlames) {
		free(m_pbCmdFlames);
		m_pbCmdFlames = NULL;
	}

    // f[^t[̔j
	if (NULL != m_pbDataFlames) {
		free(m_pbDataFlames);
		m_pbDataFlames = NULL;
	}
}


//-----------------------------------------------------------------------------
// @outline		s܂B
// @explanation	s܂B
// @param		none
// @return		ꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::Initialize()
{
	DWORD   dwRet = SW_SUCCESS;

	// R}hEt[̃m
	m_pbCmdFlames = (BYTE*)malloc(CMD_FLAME_MAX_SIZE);
	if (NULL == m_pbCmdFlames) {
		dwRet = SW_ERROR_INIT;
	}

    if (SW_SUCCESS == dwRet) {
		// f[^Et[̃m
		m_pbDataFlames = (BYTE*)malloc(DAT_FLAME_MAX_SIZE);
		if (NULL == m_pbDataFlames) {
            dwRet = SW_ERROR_INIT;
		}
	}

	if (SW_SUCCESS != dwRet) {
		// ̊mۂɎsꍇ́Amۍς݂̃B
		if (NULL != m_pbCmdFlames) {
			free(m_pbCmdFlames);
			m_pbCmdFlames = NULL;
		}

		if (NULL != m_pbDataFlames) {
			free(m_pbDataFlames);
			m_pbDataFlames = NULL;
		}
	}

    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		ResetR}hEt[쐬܂B
// @explanation	ResetR}hEt[쐬܂B
// @param       [OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeResetCommand(DWORD* dwCmdLength)
{
	DWORD	dwRet = SW_SUCCESS;
    BYTE    bLen  = 0x00;

	// NA
	ZeroMemory(m_pbCmdFlames, CMD_FLAME_MAX_SIZE);
    *dwCmdLength = 0UL;
    bLen         = 0x01;

	m_pbCmdFlames[0] = SOH;                                     // SOH
	m_pbCmdFlames[1] = bLen;                                    // LEN
	m_pbCmdFlames[2] = CMD_NUM_RESET;                           // COM
    m_pbCmdFlames[3] = getCheckSum(m_pbCmdFlames, 1, bLen + 1); // SUM
	m_pbCmdFlames[4] = ETX;                                     // ETX
	*dwCmdLength = 5UL;

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		ResetR}hEt[ɑΉXe[^XEt[`FbN܂B
// @explanation	ResetR}hEt[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	oʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckResetStatus(BYTE* pbSt1)
{
	return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		BaudRateR}hEt[쐬܂B
// @explanation	BaudRateR}hEt[쐬܂B
// @param		[IN]bBps            {[[glw肵܂B(0x00 = 115,200bps | 0x01 = 250,000bps | 0x02 = 500,000 bps | 0x03 = 1,000,000 bps)
//              [IN]bVoltage        dw肵܂B(:)3.30V -> 33 -> 0x21:dl̏ʂ܂ł̒l16iƂĎw肵܂B)
//              [OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊO Ԃ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeBaudRateCommand(BYTE bBps, BYTE bVoltage, DWORD* dwCmdLength)
{
	DWORD   dwRet = SW_SUCCESS;
    BYTE    bLen  = 0x00;
	
	// NA
	ZeroMemory(m_pbCmdFlames, CMD_FLAME_MAX_SIZE);
    *dwCmdLength    = 0UL;
    bLen            = 0x03;

    if (bBps == BPS_115200 ||
		bBps == BPS_250000 ||
		bBps == BPS_500000 ||
		bBps == BPS_1000000) {
			m_pbCmdFlames[0] = SOH;                                     // SOH
			m_pbCmdFlames[1] = bLen;                                    // LEN
			m_pbCmdFlames[2] = CMD_NUM_BAUD_RATE_SET;                   // COM
			m_pbCmdFlames[3] = bBps;								    // D01
			m_pbCmdFlames[4] = bVoltage;    						    // D02
			m_pbCmdFlames[5] = getCheckSum(m_pbCmdFlames, 1, bLen + 1); // SUM
			m_pbCmdFlames[6] = ETX;
			*dwCmdLength = 7UL;
	}
	else {
        dwRet = SW_ERROR_PARAMETER;
	}

    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		BaudRateR}hEt[ɑ΂Xe[^XEt[`FbN܂B
// @explanation	BaudRateR}hEt[ɑ΂Xe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	oʂi[܂B
//				[OUT]pbD01	16ĩf[^ƂđMA̎gɃEFCgA^CAEgݒi[܂B
//				[OUT]pbD02	vO~OE[h̐ݒi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckBaudRateStatus(BYTE* pbSt1, BYTE* pbD01, BYTE* pbD02)
{
	DWORD   dwRet	= SW_SUCCESS;
    BYTE    bLen    = 0x00;
	BYTE	bSum	= INIT_SUM;

    // LEN擾
    bLen = m_pbDataFlames[1];

	// `FbNSUM擾
	bSum = getCheckSum(m_pbDataFlames, 1, bLen + 1);

	if (bSum == m_pbDataFlames[5]) {
		if (m_pbDataFlames[0] == STX &&
			m_pbDataFlames[1] == 0x03 &&
			m_pbDataFlames[6] == ETX) {
				*pbSt1 = m_pbDataFlames[2];
				*pbD01 = m_pbDataFlames[3];
				*pbD02 = m_pbDataFlames[4];
		}
		else {
			// t[EtH[}bgs
			dwRet = SW_ERROR_FRAME_FORMAT;	
		}
	}
	else {
		// `FbNSUMs
		dwRet = SW_ERROR_CHECKSUM;
	}

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		BlockEraseR}hEt[쐬܂B
// @explanation	BlockEraseR}hEt[쐬܂B
// @param		[IN]bSal            JnAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bSam            JnAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bSah            JnAhXLow   irbg7  - rbg0j w肵܂B
//              [OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊO Ԃ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeBlockEraseCommand(BYTE bSal, BYTE bSam, BYTE bSah, DWORD* dwCmdLength)
{
	DWORD   dwRet = SW_SUCCESS;
    BYTE    bLen  = 0x00;
	
	// NA
	ZeroMemory(m_pbCmdFlames, CMD_FLAME_MAX_SIZE);
    *dwCmdLength    = 0UL;
    bLen            = 0x04;
	
	m_pbCmdFlames[0] = SOH;									    // SOH
	m_pbCmdFlames[1] = 0x04;								    // LEN
	m_pbCmdFlames[2] = CMD_NUM_BLOCK_ERASE;					    // COM
	m_pbCmdFlames[3] = bSal;								    // SAL
	m_pbCmdFlames[4] = bSam;								    // SAM
	m_pbCmdFlames[5] = bSah;							    	// SAH
	m_pbCmdFlames[6] = getCheckSum(m_pbCmdFlames, 1, bLen + 1);	// SUM
	m_pbCmdFlames[7] = ETX;									    // ETX
	*dwCmdLength = 8UL;

    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		BlockEraseR}hɑ΂Xe[^XEt[`FbN܂B
// @explanation	BlockEraseR}hɑ΂Xe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	oʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckBlockEraseStatus(BYTE* pbSt1)
{
	return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		SecurityGetR}hEt[쐬܂B
// @explanation	SecurityGetR}hEt[쐬܂B
// @param       [OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeSecurityGetCommand(DWORD* dwCmdLength)
{
	DWORD	dwRet = SW_SUCCESS;
    BYTE    bLen  = 0x00;

	// NA
	ZeroMemory(m_pbCmdFlames, CMD_FLAME_MAX_SIZE);
    *dwCmdLength = 0UL;
    bLen         = 0x01;

	m_pbCmdFlames[0] = SOH;                                     // SOH
	m_pbCmdFlames[1] = bLen;                                    // LEN
    m_pbCmdFlames[2] = CMD_NUM_SECURITY_GET;                    // COM
    m_pbCmdFlames[3] = getCheckSum(m_pbCmdFlames, 1, bLen + 1); // SUM
	m_pbCmdFlames[4] = ETX;                                     // ETX
	*dwCmdLength = 5UL;

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		SecurityGetR}hɑ΂Xe[^XEt[`FbN܂B
// @explanation	SecurityGetR}hɑ΂Xe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	oʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckSecurityGetCommand(BYTE* pbSt1)
{
    return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		SecurityGetR}h̃f[^Et[`FbN܂B
// @explanation	SecurityGetR}h̃f[^Et[`FbN܂B
// @param		[IN] 
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckSecurityGetDataFrame(BYTE* pbFlg, BYTE* pbBot, BYTE* pbSsl, BYTE* pbSsh, BYTE* pbSel, BYTE* pbSeh)
{
    DWORD   dwRet = SW_SUCCESS;
    BYTE    bLen;
    BYTE    bSum  = INIT_SUM;

    // LEN擾
    bLen = m_pbDataFlames[1];

    // `FbNSUM擾
    bSum = getCheckSum(m_pbDataFlames, 1, bLen + 1);

    // STXETXȏꍇ
    if (STX == m_pbDataFlames[0] &&
        ETX == m_pbDataFlames[bLen + 3]) {
            // SUM햼ꍇ
            if (bSum == m_pbDataFlames[bLen + 2]) {
                if (0x08 == bLen) {
                    BYTE bOffset = 2;

                    // FLG
                    memcpy_s(pbFlg, 1, m_pbDataFlames + bOffset, 1);

                    // BOT
                    bOffset +=1;
                    memcpy_s(pbBot, 1, m_pbDataFlames + bOffset, 1);
                
                    // SSL
                    bOffset += 1;
                    memcpy_s(pbSsl, 1, m_pbDataFlames + bOffset, 1);

                    // SSH
                    bOffset += 1;
                    memcpy_s(pbSsh, 1, m_pbDataFlames + bOffset, 1);

                    // SEL
                    bOffset += 1;
                    memcpy_s(pbSel, 1, m_pbDataFlames + bOffset, 1);

                    // SEH
                    bOffset += 1;
                    memcpy_s(pbSeh, 1, m_pbDataFlames + bOffset, 1);
                }
                else {
                    // t[EtH[}bgs
                    dwRet = SW_ERROR_FRAME_FORMAT;
                }
            }
            else {
                dwRet = SW_ERROR_CHECKSUM;
            }
    }
    else {
        // t[EtH[}bgs
        dwRet = SW_ERROR_FRAME_FORMAT;
    }
    
    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		ProgrammingR}hEt[쐬܂B
// @explanation	ProgrammingR}hEt[쐬܂B
// @param		[IN]bSal	݊JnAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bSam	݊JnAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bSah	݊JnAhXLow   irbg7  - rbg0j w肵܂B
//				[IN]bEal	ݏIAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bEam	ݏIAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bEah	ݏIAhXLow   irbg7  - rbg0j w肵܂B
//              [OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeProgrammingCommand(BYTE bSal, BYTE bSam, BYTE bSah, BYTE bEal, BYTE bEam, BYTE bEah, DWORD* dwCmdLength)
{
	BYTE	bCom = CMD_NUM_PROGRAMMING;

	return makeAddressRangeCommand(bCom, bSal, bSam, bSah, bEal, bEam, bEah, dwCmdLength);
}


//-----------------------------------------------------------------------------
// @outline		ProgrammingR}hɑ΂Xe[^XEt[`FbN܂B
// @explanation	ProgrammingR}hɑ΂Xe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	oʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckProgrammingStatus(BYTE* pbSt1)
{
	return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		ProgrammingR}h̃f[^Et[쐬܂B
// @explanation	ProgrammingR}h̃f[^Et[쐬܂B
// @param		[IN]  pbWriteData   ރ[UEvOw肵܂B
//				[IN]  nLength       ރ[UEvO̒w肵܂B
//				[IN]  fFinalFrame   ŏIt[̏ꍇɂ TRUE A ŏIt[ȊȌꍇɂ FALSE w肵܂B
//              [OUT] dwCmdLength   R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeProgrammingDataFrame(BYTE* pbWriteData, UINT nLength, BOOL fFinalFrame, DWORD* dwCmdLength)
{
	return makeDataFrame(pbWriteData, nLength, fFinalFrame, dwCmdLength);
}


//-----------------------------------------------------------------------------
// @outline		ProgrammingR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @explanation	ProgrammingR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	f[^MmFʂi[܂B
//				[OUT]pbSt2	݌ʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckProgrammingDataFrameStatus(BYTE* pbSt1, BYTE* pbSt2)
{
	return checkDataFrame(pbSt1, pbSt2);
}


//-----------------------------------------------------------------------------
// @outline		Sf[^]ɑ΂Xe[^XEt[`FbN܂B
// @explanation	Sf[^]ɑ΂Xe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	xt@Cʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckProgrammingDataFrameStatus(BYTE* pbSt1)
{
	return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		VerifyR}hEt[쐬܂B
// @explanation	VerifyR}hEt[쐬܂B
// @param		[IN]bSal	xt@CJnAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bSam	xt@CJnAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bSah	xt@CJnAhXLow   irbg7  - rbg0j w肵܂B
//				[IN]bEal	xt@CIAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bEam	xt@CIAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bEah	xt@CIAhXLow   irbg7  - rbg0j w肵܂B
//              [OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeVerifyCommand(BYTE bSal, BYTE bSam, BYTE bSah, BYTE bEal, BYTE bEam, BYTE bEah, DWORD* dwCmdLength)
{
	BYTE	bCom = CMD_NUM_VERIFY;

	return makeAddressRangeCommand(bCom, bSal, bSam, bSah, bEal, bEam, bEah, dwCmdLength);
}


//-----------------------------------------------------------------------------
// @outline		VerifyR}hɑ΂Xe[^XEt[`FbN܂B
// @explanation	VerifyR}hɑ΂Xe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	R}hMʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckVerifyStatus(BYTE* pbSt1)
{
	return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		VerifyR}h̃f[^Et[쐬܂B
// @explanation	VerifyR}h̃f[^Et[쐬܂B
// @param		[IN]  pbVerifyData  xt@Cs[UEvOw肵܂B
//				[IN]  nLength       xt@Cs[UEvO̒w肵܂B
//				[IN]  fFinalFrame   ŏIt[̏ꍇɂ TRUE A ŏIt[ȊȌꍇɂ FALSE w肵܂B
//              [OUT] dwCmdLength   R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeVerifyDataFrame(BYTE* pbVerifyData, UINT nLength, BOOL fFinalFrame, DWORD* dwCmdLength)
{
	return makeDataFrame(pbVerifyData, nLength, fFinalFrame, dwCmdLength);
}


//-----------------------------------------------------------------------------
// @outline		VerifyR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @explanation	VerifyR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	f[^MmFʂi[܂B
//				[OUT]pbSt2	xt@Cʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckVerifyDataFrameStatus(BYTE* pbSt1, BYTE* pbSt2)
{
	return checkDataFrame(pbSt1, pbSt2);
}


//-----------------------------------------------------------------------------
// @outline		ChecksumR}hEt[쐬܂B
// @explanation	ChecksumR}hEt[쐬܂B
// @param		[IN]bSal	`FbNTvZJnAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bSam	`FbNTvZJnAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bSah	`FbNTvZJnAhXLow   irbg7  - rbg0j w肵܂B
//				[IN]bEal	`FbNTvZIAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bEam	`FbNTvZIAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bEah	`FbNTvZIAhXLow   irbg7  - rbg0j w肵܂B
//              [OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeChecksumCommand(BYTE bSal, BYTE bSam, BYTE bSah, BYTE bEal, BYTE bEam, BYTE bEah, DWORD* dwCmdLength)
{
    BYTE	bCom = CMD_NUM_CHECKSUM;

	return makeAddressRangeCommand(bCom, bSal, bSam, bSah, bEal, bEam, bEah, dwCmdLength);
}


//-----------------------------------------------------------------------------
// @outline		ChecksumR}hɑ΂Xe[^XEt[`FbN܂B
// @explanation	ChecksumR}hɑ΂Xe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	R}hMʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckChecksumStatus(BYTE* pbSt1)
{
    return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		ChecksumR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @explanation	ChecksumR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbCk1	CK1(`FbNTEf[^̉8rbg)i[܂B
//              [OUT]pbCk2  CK2(`FbNTEf[^̏8rbg)i[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckChecksumDataFrams(BYTE* pbCk1, BYTE* pbCk2)
{
    DWORD   dwRet = SW_SUCCESS;
    BYTE    bLen;
    BYTE    bSum  = INIT_SUM;

    // LEN擾
    bLen = m_pbDataFlames[1];

    // `FbNSUM擾
    bSum = getCheckSum(m_pbDataFlames, 1, bLen + 1);

    // STXETX햼ꍇ
    if (STX == m_pbDataFlames[0] &&
        ETX == m_pbDataFlames[bLen + 3]) {
            // SUM햼ꍇ
            if (bSum == m_pbDataFlames[bLen + 2]) {
                // LENȏꍇ
                if (0x02 == bLen) {
                    BYTE bOffset = 2;

                    // CK1(`FbNTEf[^̉8rbg)
                    *pbCk1 = m_pbDataFlames[2];

                    // CK2(`FbNTEf[^̏8rbg)
                    *pbCk2 = m_pbDataFlames[3];
                }
                else {
                    // t[EtH[}bgs
                    dwRet = SW_ERROR_FRAME_FORMAT;
                }
            }
            else {
                dwRet = SW_ERROR_CHECKSUM;
            }
    }
    else {
        // t[EtH[}bgs
        dwRet = SW_ERROR_FRAME_FORMAT;
    }
    
    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		SiliconSignatureR}hEt[쐬܂B
// @explanation	SiliconSignatureR}hEt[쐬܂B
// @param		[OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeSiliconSignatureCommand(DWORD* dwCmdLength)
{
	DWORD	dwRet = SW_SUCCESS;
    BYTE    bLen    = 0x00;

	// NA
	ZeroMemory(m_pbCmdFlames, CMD_FLAME_MAX_SIZE);
    *dwCmdLength = 0UL;
    bLen         = 0x01;

	m_pbCmdFlames[0] = SOH;                                     // SOH
	m_pbCmdFlames[1] = bLen;                                    // LEN
    m_pbCmdFlames[2] = CMD_NUM_SILICON_SIGNATURE;               // COM
	m_pbCmdFlames[3] = getCheckSum(m_pbCmdFlames, 1, bLen + 1); // SUM
	m_pbCmdFlames[4] = ETX;                                     // ETX
	*dwCmdLength     = 5L;

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		SiliconSignatureR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @explanation	SiliconSignatureR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	f[^MmFʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckSiliconSignatureStatus(BYTE* pbSt1)
{
    return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		SiliconSignatureR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @explanation	SiliconSignatureR}h̃f[^Et[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbDec	foCXER[hi[܂B
//              [OUT]pbDev  foCXi[܂BiI[tj
//              [OUT]pbCen  R[hEtbVROMŏIAhXi[܂B
//              [OUT]pbDen  f[^EtbVROMŏIAhX
//              [OUT]pbVer  
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckSiliconSignatureDataFrames(BYTE* pbDec, BYTE* pbDev,BYTE* pbCen, BYTE* pbDen, BYTE* pbVer)
{
    DWORD   dwRet = SW_SUCCESS;
    BYTE    bLen;
    BYTE    bSum  = INIT_SUM;

    // LEN擾
    bLen = m_pbDataFlames[1];

    // `FbNSUM擾
    bSum = getCheckSum(m_pbDataFlames, 1, bLen + 1);

    // STXETX햼ꍇ
    if (STX == m_pbDataFlames[0] &&
        ETX == m_pbDataFlames[bLen + 3]) {
            // SUM햼ꍇ
            if (bSum == m_pbDataFlames[bLen + 2]) {
                // LENȏꍇ
                if (0x16 == bLen) {
                    BYTE bOffset = 2;

                    // DEC(foCXR[h)
                    memcpy_s(pbDec, 3, m_pbDataFlames + bOffset, 3);

                    // DEV(foCX)
                    bOffset +=3;
                    memcpy_s(pbDev, 10, m_pbDataFlames + bOffset, 10);
                    pbDev[11] = NULL;
                
                    // CEN(R[hEtbVROMŏIAhX)
                    bOffset += 10;
                    memcpy_s(pbCen, 3, m_pbDataFlames + bOffset, 3);

                    // DEN(f[^EtbVROMŏIAhX)
                    bOffset += 3;
                    memcpy_s(pbDen, 3, m_pbDataFlames + bOffset, 3);

                    // VER(t@[EFAEo[W)
                    bOffset += 3;
                    memcpy_s(pbVer, 3, m_pbDataFlames + bOffset, 3);
                }
                else {
                    // t[EtH[}bgs
                    dwRet = SW_ERROR_FRAME_FORMAT;
                }
            }
            else {
                dwRet = SW_ERROR_CHECKSUM;
            }
    }
    else {
        // t[EtH[}bgs
        dwRet = SW_ERROR_FRAME_FORMAT;
    }
    
    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		BlockBlankCheckR}hEt[쐬܂B
// @explanation	BlockBlankCheckR}hEt[쐬܂B
// @param		[IN]bSal            ubNEuNE`FbNJnAhX Low    (rbg 7 - rbg 0)
//              [IN]bSam            ubNEuNE`FbNJnAhX Middle (rbg15 - rbg 8)
//              [IN]bSah            ubNEuNE`FbNJnAhX High   (rbg23 - rbg16)
//              [IN]bEal            ubNEuNE`FbNIAhX Low    (rbg 7 - rbg 0)
//              [IN]bEam            ubNEuNE`FbNIAhX Middle (rbg15 - rbg 8)
//              [IN]bEah            ubNEuNE`FbNIAhX High   (rbg23 - rbg16)
//              [IN]bD01            uNE`FbNẅi0x00:w肵ubN | 0x01:w肵ubNƃtbVEIvVj
//      		[OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::MakeBlockBlankCheckCommand(BYTE bSal, BYTE bSam, BYTE bSah, BYTE bEal, BYTE bEam, BYTE bEah, BYTE bD01, DWORD* dwCmdLength)
{
	DWORD	dwRet = SW_SUCCESS;
    BYTE    bLen    = 0x00;

	// NA
	ZeroMemory(m_pbCmdFlames, CMD_FLAME_MAX_SIZE);
    *dwCmdLength = 0UL;
    bLen         = 0x08;

	m_pbCmdFlames[0]  = SOH;                                     // SOH
	m_pbCmdFlames[1]  = bLen;                                    // LEN
    m_pbCmdFlames[2]  = CMD_NUM_BLOCK_BLANK_CHECK;               // COM
    m_pbCmdFlames[3]  = bSal;                                    // SAL
    m_pbCmdFlames[4]  = bSam;                                    // SAM
    m_pbCmdFlames[5]  = bSah;                                    // SAH
    m_pbCmdFlames[6]  = bEal;                                    // EAL
    m_pbCmdFlames[7]  = bEam;                                    // EAM
    m_pbCmdFlames[8]  = bEah;                                    // EAH
    m_pbCmdFlames[9]  = bD01;                                    // D01
	m_pbCmdFlames[10] = getCheckSum(m_pbCmdFlames, 1, bLen + 1); // SUM
	m_pbCmdFlames[11] = ETX;                                     // ETX
	*dwCmdLength      = 12UL;

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		BlockBlankCheckR}hEt[ɑΉXe[^XEt[`FbN܂B
// @explanation	BlockBlankCheckR}hEt[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	oʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckBlockBlankCheckStatus(BYTE* pbSt1)
{
    return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		AhX͈͂܂ރR}hEt[쐬܂B
// @explanation	AhX͈͂܂ރR}hEt[쐬܂B
// @param		[IN]bCom	R}hԍw肵܂B
//				[IN]bSal	JnAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bSam	JnAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bSah	JnAhXLow   irbg7  - rbg0j w肵܂B
//				[IN]bEal	IAhXHigh  irbg23 - rbg16jw肵܂B
//				[IN]bEam	IAhXMiddleirbg15 - rbg8j w肵܂B
//				[IN]bEah	IAhXLow   irbg7  - rbg0j w肵܂B
//      		[OUT]dwCmdLength    R}hEt[̃TCYi[܂B
// @return		쐬ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::makeAddressRangeCommand(BYTE bCom, BYTE bSal, BYTE bSam, BYTE bSah, BYTE bEal, BYTE bEam, BYTE bEah, DWORD* dwCmdLength)
{
    DWORD   dwRet = SW_SUCCESS;

	// NA
	ZeroMemory(m_pbCmdFlames, CMD_FLAME_MAX_SIZE);
    *dwCmdLength = 0UL;

	m_pbCmdFlames[0]  = SOH;                                // SOH
	m_pbCmdFlames[1]  = 0x07;                               // LEN
	m_pbCmdFlames[2]  = bCom;                               // COM
	m_pbCmdFlames[3]  = bSal;                               // SAL
	m_pbCmdFlames[4]  = bSam;                               // SAM
	m_pbCmdFlames[5]  = bSah;                               // SAH
	m_pbCmdFlames[6]  = bEal;                               // EAL
	m_pbCmdFlames[7]  = bEam;                               // EAM
	m_pbCmdFlames[8]  = bEah;                               // EAH
	m_pbCmdFlames[9]  = getCheckSum(m_pbCmdFlames, 1, 8);   // SUM
	m_pbCmdFlames[10] = ETX;                                // ETX
	*dwCmdLength      = 11UL;

	return dwRet;
}



//-----------------------------------------------------------------------------
// @outline		eR}hEt[ɑΉXe[^XEt[`FbN܂B
// @explanation	^[Qbgُ̈퉞ȂǂŁAf[^e܂܂Ȃꍇ̃Xe[^XEt[`FbN܂B
//              :)BaudRateStatusݒR}ḧُ퉞ɂ͂̃tH[}bgŃXe[^XEt[ԂB
// @param		[OUT]pbSt1	oʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::CheckStatus(BYTE* pbSt1) 
{
    return checkStatus(pbSt1);
}


//-----------------------------------------------------------------------------
// @outline		R}hEt[ɑΉXe[^XEt[`FbN܂B
// @explanation	R}hEt[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	oʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::checkStatus(BYTE* pbSt1)
{
	DWORD	dwRet	= SW_SUCCESS;
	BYTE	bSum	= INIT_SUM;
    BYTE    bLen    = 0x00;

    // LEN擾
    bLen = m_pbDataFlames[1];

	// `FbNSUM擾
    bSum = getCheckSum(m_pbDataFlames, 1, bLen + 1);
	if (m_pbDataFlames[0] == STX &&
		m_pbDataFlames[1] == 0x01 &&
		m_pbDataFlames[4] == ETX) {
			if (bSum == m_pbDataFlames[3]) {
				*pbSt1 = m_pbDataFlames[2];	// ST1
			}
			else {
				// `FbNSUMs
				dwRet = SW_ERROR_CHECKSUM;
			}
	}
	else {
		// t[EtH[}bgs
		dwRet = SW_ERROR_FRAME_FORMAT;
	}

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		f[^Et[쐬܂B
// @explanation	f[^Et[쐬܂B
// @param		[IN]  pbData        f[^w肵܂B
//				[IN]  nLength       f[^̒w肵܂B
//				[IN]  fFinalFrame   ŏIt[̏ꍇɂ TRUE A ŏIt[ȊȌꍇɂ FALSE w肵܂B
//              [OUT] dwCmdLength   R}hEt[̃TCYi[܂B  
// @return		쐬ɐꍇɂ SW_SUCCESS As SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::makeDataFrame(BYTE* pbData, UINT nLength, BOOL fFinalFrame, DWORD* dwCmdLength)
{
    DWORD   dwRet = SW_SUCCESS;

	// NA
	ZeroMemory(m_pbCmdFlames, CMD_FLAME_MAX_SIZE);
    *dwCmdLength = 0UL;

    // f[^̃TCY`FbN
    if (DAT_DATA_MAX_SIZE < nLength) {
        dwRet = SW_ERROR_PARAMETER;
    }
    else {
        // Datäփf[^Rs[B
        memcpy_s(m_pbCmdFlames + 2, DAT_DATA_MAX_SIZE, pbData, nLength);

		m_pbCmdFlames[0]           = STX;										    // STX
		m_pbCmdFlames[1]           = nLength;									    // LEN
		m_pbCmdFlames[2 + nLength] = getCheckSum(m_pbCmdFlames, 1, nLength + 1);    // SUM
		m_pbCmdFlames[3 + nLength] = fFinalFrame == FALSE ? ETB : ETX;			    // ETB/ETX
		*dwCmdLength               = nLength + 4UL;
    }

    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		f[^Et[ɑΉXe[^XEt[`FbN܂B
// @explanation	f[^Et[ɑΉXe[^XEt[`FbN܂B
// @param		[OUT]pbSt1	ST1̌ʂi[܂B
//				[OUT]pbSt2	ST2̌ʂi[܂B
// @return		`FbNɊꍇɂ SW_SUCCESS AɊłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CCommandProcessor::checkDataFrame(BYTE* pbSt1, BYTE* pbSt2)
{
	DWORD   dwRet   = SW_SUCCESS;
	BYTE    bSum    = INIT_SUM;

	// `FbNSUM擾
	bSum = getCheckSum(m_pbDataFlames, 1, 3);
	if (m_pbDataFlames[0] == STX &&
		m_pbDataFlames[1] == 0x02 &&
		m_pbDataFlames[5] == ETX) {
			if (bSum == m_pbDataFlames[4]) {
				*pbSt1 = m_pbDataFlames[2];	// ST1
				*pbSt2 = m_pbDataFlames[3];	// ST2
			}
			else {
				// `FbNSUMs
				dwRet = SW_ERROR_CHECKSUM;
			}
	}
	else {
		// t[EtH[}bgs
		dwRet = SW_ERROR_FRAME_FORMAT;
	}

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		`FbNSUM擾܂B
// @explanation	`FbNSUM擾܂B
// @param		[IN]bFrames		R}hEt[A܂̓f[^Et[w肵܂B
//				[IN]nStartIndex vZvf̊Jnʒuw肵܂B
//				[IN]nCount		vZvf̌w肵܂B
// @return		vZ`FbNSUMԂ܂B
//-----------------------------------------------------------------------------
BYTE CCommandProcessor::getCheckSum(BYTE* bFrames, int nStartIndex, int nCount)
{
	BYTE	bSum;

	// lZbg
	bSum = INIT_SUM;
	for (int i = nStartIndex; i < (nStartIndex + nCount); i++) {
		bSum -= bFrames[i];
	}

	return bSum;
}
