﻿/*
 * Copyright (c) 2012,2015 Renesas Electronics Corporation and TESSERA
 * TECHNOLOGY INC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//=============================================================================
// インクルード
//=============================================================================
#include "Common.h"
#include "res\resource.h"
#include "SerialPortProcessor.h"
#include "SequenceProcessor.h"
#include "SequenceProcessorForMot.h"
#include <Dbt.h>

#include <MMSystem.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//=============================================================================
// リンク
//=============================================================================
#pragma comment(lib, "version.lib") // for バージョン情報

//=============================================================================
// 定数
//=============================================================================
// システム・メニューのバージョン情報アイテム
// MEMO:システム コマンドの範囲内になければなりません。
#define IDM_ABOUTBOX            0x0010                                          

// シリアルモニタ用タイマID(0x01)
#define MONITOR_TIMER_ID		0x01

// シリアルモニタ用タイマインターバル(100msec)
#define	MONITOR_TIMER_INTERVAL	100

// ログ出力するメッセージ量の最大サイズ = 64kbyte
#define	MONITOR_MAX_LOGSIZE		(64 * 1024)

// シリアルモニタに設定できるボーレートの下限値
#define MIN_BAUDRATE			110

// シリアルモニタに設定できるボーレートの上限値
#define	MAX_BAUDRATE			921600
//=============================================================================
// マクロ
//=============================================================================
// 画面領域の取得
#define GetMonitorRect(rc)      SystemParametersInfo(SPI_GETWORKAREA, 0, rc, 0)

//=============================================================================
// 関数宣言
//=============================================================================
// メインダイアログのコールバック関数です。
BOOL    CALLBACK MainDialogProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

// メインダイアログの初期化を行います。
BOOL    InitMainDialog(HWND hWnd, WPARAM wParam, LPARAM lParam);

// バージョン情報ダイアログのコールバック関数です。
BOOL    CALLBACK AboutDialogProc(HWND, UINT, WPARAM, LPARAM);

// バージョン情報ダイアログの初期化を行います。
BOOL    InitAboutDialog(HWND hWnd, WPARAM wParam, LPARAM lParam);

// 進捗ダイアログのコールバック関数です。
BOOL    CALLBACK ProgressDlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

// ファイルエディット(IDC_EDIT_DROP)のサブクラス化されたコールバック関数です。
LRESULT CALLBACK EditDropSubWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

// F/W更新ログ(IDC_EDIT_UPDATE_LOG)のサブクラス化されたコールバック関数です。
LRESULT CALLBACK EditUpdateLogSubWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

// メッセージボックス表示前のフックプロシージャ関数です。
LRESULT CALLBACK CBTActivateMessageBox(int, WPARAM, LPARAM);

// ファイルのバージョン番号を取得します。
void    GetFileVersion(TCHAR *version, INT iLen);

// シリアルポート情報の初期化を行います。
void    InitializeSerialPortInfo(HWND hWnd);

// シリアルポート選択コンボボックスのドロップダウンメッセージを処理します。
void    OnCbnDropdownComboSerialPoprt(HWND hWnd);

// シリアルポート選択コンボボックスの選択アイテム変更メッセージを処理します。
void    OnCbnSelchangeComboSerialPoprt(HWND hWnd);

// ボーレート選択コンボボックスの初期化を行います。
void    InitializeBaudRate(HWND hWnd);

// シリアルモニタ用　ボーレート選択コンボボックスの初期化を行います。
void	InitializeMonitorBaudRate(HWND hWnd);

// ファイル選択ボタンのクリックメッセージを処理します。
void    OnBnClickedButtonFileSelect(HWND hWnd);

// 書き込みボタンのクリックメッセージを処理します。
void    OnBnClickedButtonWrite(HWND hWnd);

// シリアルモニタ用オープンボタンのクリックメッセージを処理します
void OnBnClickedButtonMonitorOpen(HWND hWnd);

// 実行結果メッセージを処理します。
void    OnResult(HWND hWnd);

// デバイス変更通知のメッセージを処理します。
void    OnDeviceChange(WPARAM wParam, LPARAM lParam);

// ツールヒント用 TTN_NEEDTEXT 通知のメッセージを処理します。
void OnNeedText(HWND hWnd, NMHDR* pNMHDR);

// ウィンドウを画面または親ウィンドウの中央へ移動します。
BOOL    CenterWindow(HWND hWnd);

// シリアルモニタ用　ポート切断
void MonitorPortClose(HWND hWnd);

// シリアルモニタ用　ポートオープン
DWORD MonitorPortOpen(HWND hWnd);

// コントロールの表示設定　シリアルモニタ用
void SetControlEnabledModeMonitor(HWND hWnd, bool isEnabled);

// シリアルモニタ用　タイマイベント
VOID CALLBACK MonitorTimerProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime);

// シリアルモニタ用　タイマ開始処理
UINT MonitorTimerStart(HWND hWnd);

// シリアルモニタ用　タイマ終了処理
void MonitorTimerStop(HWND hWnd);

// DTRの状態確認
DWORD DtrCheck(HWND hWnd);

// UART通信設定のボー・レート設定および電圧設定データの設定を行います。
DWORD ComBaudRateSetProc(BYTE bBaudRate, BYTE bVoltage);

// 指定された時間だけウェイトします。
DWORD Wait(DWORD dwMilisecond);

// データの受信待ちを行います。
DWORD WaitStatusFrames(DWORD* dwLength, DWORD dwTimeout);

//=============================================================================
// グローバル変数
//=============================================================================
// アプリケーションのインスタンス
HINSTANCE               g_hInstance;

// シリアルポート情報のリスト
vector<SERIALPORTINFO>  g_SerialPortInfoList;

// シリアルポートコンボボックスの選択されたインデックス
INT                     g_SerialportSelectedIndex = 0;

// コンテキスト構造体
CONTEXTINFO             g_ContextInfo;

// 書き込み結果構造体
RESULTINFO              g_ResultInfo;

// CSequenceProcessorクラス
CSequenceProcessor			g_SequenceProcessor;

// CSequenceProcessorFormクラス
CSequenceProcessorForMot	g_SequenceProcessorForMot;

// 進捗ダイアログのウィンドウハンドル
HWND                    g_hWndProgressDlg;

// ファイルエディット(IDC_EDIT_DROP)の本来のウィンドウプロシージャ
WNDPROC                 g_wndProcEditDrop;

// メッセージボックス表示前のフックプロシージャ
HHOOK hCBTHook = 0;

DEVCHANGEINFO           g_DevChangeInfo;

// F/W更新ログ(IDC_EDIT_UPDATE_LOG)の本来のウィンドウプロシージャ
WNDPROC                 g_wndProcEditUpdateLog;

// リッチテキストのモジュールハンドル
HMODULE                 g_hRtLib;

// リッチテキストボックスのウィンドウハンドル
HWND                    g_hEditUpdateLog;

// Monitor用にシリアルポートを開いているかどうかの確認
bool					g_isMonitorOpen;

// Monitor用シリアルポート制御クラス
CSerialPortProcessor	g_MonitorComPort;

// DTRの初期値
bool					g_isInvertDTR;

//-----------------------------------------------------------------------------
// @outline     Win32アプリケーションの初期エントリポイントです。
// @explanation Win32アプリケーションの初期エントリポイントです。Windowsシステムが呼び出します。
// @param       [IN] hInstance      現在のインスタンスのハンドルを指定します。
//              [IN] hPrevInstance  以前のインスタンスのハンドルを指定します。
//              [IN] lpCmdLine      コマンドラインを指定します。
//              [IN] nCmdShow       表示状態を指定します。
// @return      終了コードを返します。
//-----------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR lpCmdLine, int nCmdShow)
{
    // インスタンスを保存
    g_hInstance = hInstance;

    // メインダイアログをモーダルダイアログとして表示
    DialogBox(hInstance, MAKEINTRESOURCE(IDD_SIMPLEWRITER_DIALOG), NULL, MainDialogProc);
    
    return 0;
}


//-----------------------------------------------------------------------------
// @outline     メインダイアログのコールバック関数です。
// @explanation メインダイアログのメッセージを処理します。
// @param       [IN] hWnd   ダイアログボックスのハンドルを指定します。
//              [IN] msg    メッセージを指定します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return      メッセージを処理した場合には TRUE 以外を、処理しなかった場合には FALSE を返します。
//-----------------------------------------------------------------------------
BOOL CALLBACK MainDialogProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    static LPTOOLTIPTEXT lptip;

    switch (msg) {
        // ダイアログの初期化
        case WM_INITDIALOG:
            return InitMainDialog(hWnd, wParam, lParam);
        
        // システムコマンド
        case WM_SYSCOMMAND:
            // バージョン情報
            if (LOWORD(wParam) == IDM_ABOUTBOX) {
                // バージョン情報ダイアログの表示
                DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_ABOUTBOX), hWnd, AboutDialogProc);
            }
            break;
        
        // コモンコントロールからの通知
        case WM_NOTIFY:
            // 通知判別
            switch (((LPNMHDR)lParam)->code) {
                // ツールチップ表示要求
                case TTN_NEEDTEXT:
                    OnNeedText(hWnd, (LPNMHDR)lParam);                    
                    break;
            }
            break;
        
        // クローズ
        case WM_CLOSE:
            // ライブラリの開放
            if (g_hRtLib != NULL) {
                FreeLibrary(g_hRtLib);
            }

			// ウィンドウが閉じるタイミングで、ログテキストへのメッセージ送信を終了する
			g_wndProcEditUpdateLog = NULL;

            // ダイアログを終了
            EndDialog(hWnd, IDOK);
            return TRUE;

        // ユーザメッセージ：実行結果
        case WM_USER_RESULT:
            OnResult(hWnd);
            break;

        // デバイス変更メッセージ
        case WM_DEVICECHANGE:
            OnDeviceChange(wParam, lParam);
            break;
        // コマンド
        case WM_COMMAND:
            // メッセージ送信元のコントロール判別
            switch (GET_WM_COMMAND_ID(wParam, lParam)) {
                // シリアルポート選択コンボボックス
                case IDC_COMBO_SERIAL_POPRT:
                    // イベントメッセージの判別
                    switch (HIWORD(wParam)) {
                        // ドロップダウンリストが表示された
                        case CBN_DROPDOWN:
                            OnCbnDropdownComboSerialPoprt(hWnd);
                            break;

                        // 選択アイテムが変更された
                        case CBN_SELCHANGE:
                            OnCbnSelchangeComboSerialPoprt(hWnd);
                            break;
                    }
                    break;

                // 更新ボタンがクリックされた
                case IDC_BUTTON_UPDATE_SERIALPORT:
                    InitializeSerialPortInfo(hWnd);
                    break;

                // 選択ボタンがクリックされた
                case IDC_BUTTON_FILE_SELECT:
                    OnBnClickedButtonFileSelect(hWnd);
                    break;

                // 書き込みボタンがクリックされた
                case IDC_BUTTON_WRITE:
                    OnBnClickedButtonWrite(hWnd);
                    break;

				// シリアルモニタのオープンボタンがクリックされた
				case IDC_BUTTON_MONITOR_OPEN:
					OnBnClickedButtonMonitorOpen(hWnd);
					break;
            }
    }

    return FALSE;
}


//-----------------------------------------------------------------------------
// @outline     メインダイアログの初期化を行います。
// @explanation メインダイアログの初期化を行います。
// @param       [IN] hWnd   ダイアログボックスのハンドルを指定します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return      メッセージを処理した場合には TRUE 以外を、処理しなかった場合には FALSE を返します。
//-----------------------------------------------------------------------------
BOOL InitMainDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
    HICON           hIcon;      // アイコンのハンドル
    HMENU           sysMenu;    // システム・メニューのハンドル
    MENUITEMINFO    mii;        // MENUITEMINFO構造体
    TCHAR           buff[260];

    // アイコンのセット
    // MEMO:アイコンをセットしないとシステム・メニューの表示ができない。
    hIcon = (HICON)LoadImage(g_hInstance, MAKEINTRESOURCE(IDR_MAINFRAME), IMAGE_ICON, 16, 16, 0);
    SendMessage(hWnd, WM_SETICON, ICON_SMALL, (LPARAM)hIcon);
     

    // MENUITEMINFO構造体の初期化
    ZeroMemory(&mii, sizeof(MENUITEMINFO));
	mii.cbSize = sizeof(MENUITEMINFO);
    
    // システム・メニュー取得
    sysMenu = GetSystemMenu(hWnd, FALSE);

    // セパレータ追加
	mii.fMask = MIIM_FTYPE;
	mii.fType = MFT_SEPARATOR;
    InsertMenuItem(sysMenu, 3, TRUE, &mii);
    
    // バージョン情報追加
    // 表示テキスト取得
    LoadString(g_hInstance, IDS_ABOUTBOX, buff, sizeof(buff));
    mii.fMask 		= MIIM_TYPE | MIIM_ID;
	mii.wID 		= IDM_ABOUTBOX;
	mii.fType 		= MFT_STRING;
	mii.dwTypeData 	= buff;    
    // バージョン情報
    InsertMenuItem(sysMenu, 4, TRUE, &mii);

    // ダイアログを中央位置へ移動
    CenterWindow(hWnd);

    // ファイルエディットのサブクラス化
    // 元のウィンドウプロシージャのアドレス取得
    g_wndProcEditDrop = (WNDPROC)(LONG_PTR)GetWindowLong(GetDlgItem(hWnd, IDC_EDIT_DROP), GWL_WNDPROC);
    
    // サブクラス化されたウィンドウプロシージャのアドレスをセット
    SetWindowLong(GetDlgItem(hWnd, IDC_EDIT_DROP), GWL_WNDPROC, (LONG)(LONG_PTR)EditDropSubWndProc);

    // ドロップの許可
    DragAcceptFiles(GetDlgItem(hWnd, IDC_EDIT_DROP), TRUE);   

    // シリアルポートの初期化
    InitializeSerialPortInfo(hWnd);

    // ボーレートコンボボックスの初期化
    InitializeBaudRate(hWnd);

	// ボーレートコンボボックスの初期化
    InitializeMonitorBaudRate(hWnd);

    // シーケンスクラスの初期化
    g_SequenceProcessor.Initialize();
	g_SequenceProcessorForMot.Initialize();

    // FTDIチェックボックスをOFF
    SendMessage(GetDlgItem(hWnd, IDC_CHECK_FTDI),  BM_SETCHECK, BST_UNCHECKED, 0);

    // シリアルポート選択コンボボックスへツールチップ追加
    HWND hWndTip = CreateWindowEx(NULL, TOOLTIPS_CLASS, NULL,
                                  WS_POPUP |TTS_ALWAYSTIP,
                                  CW_USEDEFAULT, CW_USEDEFAULT,
                                  CW_USEDEFAULT, CW_USEDEFAULT,
                                  hWnd,
                                  NULL, 
                                  g_hInstance,
                                  NULL);

    TOOLINFO ti;
    ZeroMemory(&ti, sizeof(TOOLINFO));
    ti.cbSize   = sizeof(TOOLINFO);
    ti.uFlags   = TTF_IDISHWND | TTF_SUBCLASS;
    ti.hwnd     = hWnd;
    ti.hinst    = g_hInstance;
    ti.uId      = (UINT_PTR)GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT);
    ti.lpszText = LPSTR_TEXTCALLBACK;
    SendMessage(hWndTip, TTM_ADDTOOL, 0, (LPARAM)&ti);

    // F/W更新ログ用リッチテキストボックス生成
    // ウィンドウサイズ
    RECT w_rect;
    GetWindowRect(hWnd, &w_rect);
    int w_width = w_rect.right - w_rect.left;   // ウィンドウ幅
    int w_height = w_rect.bottom - w_rect.top;  // ウィンドウ高
    // クライアントサイズ
    RECT c_rect;
    GetClientRect(hWnd, &c_rect);
    int c_width = c_rect.right - c_rect.left;   // クライアント幅
    int c_height = c_rect.bottom - c_rect.top;  // クライアント高
	int	pointY = 195;
    g_hRtLib = NULL;
    g_hRtLib = LoadLibrary(_T("RICHED32.DLL"));
    g_hEditUpdateLog = CreateWindowEx(WS_EX_CLIENTEDGE,
        _T("RICHEDIT"),
        _T(""),
        WS_CHILD | WS_VISIBLE | WS_BORDER | ES_MULTILINE |
        WS_VSCROLL | WS_HSCROLL | ES_AUTOVSCROLL | ES_AUTOHSCROLL | ES_READONLY,
        15,			// X (固定)
        pointY,		// Y (上端座標、Heightも変更すること)
        w_width  - (w_width  - c_width) - 30,			// Width  (ウィンドウ幅 - 枠幅)
        w_height - (w_height - c_height) - pointY - 10,	// Height (ウィンドウ高 - 枠幅 - Y)
        hWnd,
        (HMENU)IDC_EDIT_UPDATE_LOG,
        g_hInstance,
        NULL);
    // フォントを選択
    SendMessage(
        g_hEditUpdateLog,
        WM_SETFONT,
        (WPARAM)SendMessage(GetDlgItem(hWnd, IDC_EDIT_DROP), WM_GETFONT, 0, 0),
        MAKELPARAM(FALSE, 0));

    // F/W更新ログのサブクラス化
    // 元のウィンドウプロシージャのアドレス取得
    g_wndProcEditUpdateLog = (WNDPROC)(LONG_PTR)GetWindowLong(g_hEditUpdateLog, GWL_WNDPROC);
    
    // サブクラス化されたウィンドウプロシージャのアドレスをセット
    SetWindowLong(g_hEditUpdateLog, GWL_WNDPROC, (LONG)(LONG_PTR)EditUpdateLogSubWndProc);

    // ウィンドウの更新
    UpdateWindow(hWnd);

    return FALSE;
}


//-----------------------------------------------------------------------------
// @outline     バージョン情報ダイアログのコールバック関数です。
// @explanation バージョン情報ダイアログのメッセージを処理します。
// @param       [IN] hWnd   ダイアログボックスのハンドルを指定します。
//              [IN] msg    メッセージを指定します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return      メッセージを処理した場合には TRUE 以外を、処理しなかった場合には FALSE を返します。
//-----------------------------------------------------------------------------
BOOL CALLBACK AboutDialogProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    switch (msg) 
    {
        // ダイアログの初期化
        case WM_INITDIALOG:
            return InitAboutDialog(hWnd, wParam, lParam);

        // クローズ
        case WM_CLOSE:
            // ダイアログを終了
            EndDialog(hWnd, IDOK);
            return TRUE;

        // コマンド
        case WM_COMMAND:
            // メッセージ送信元のコントロール判別
            switch (GET_WM_COMMAND_ID(wParam, lParam)) {
                // OKボタン
                case IDOK:
                    EndDialog(hWnd, IDOK);
                    return TRUE;
            }
            break;
    }

    return FALSE;
}

//-----------------------------------------------------------------------------
// @outline     バージョン情報ダイアログの初期化を行います。
// @explanation バージョン情報ダイアログの初期化を行います。
// @param       [IN] hWnd   ダイアログボックスのハンドルを指定します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return      メッセージを処理した場合には TRUE 以外を、処理しなかった場合には FALSE を返します。
//-----------------------------------------------------------------------------
BOOL InitAboutDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
    TCHAR   version[16];

    // バージョン番号を取得
    GetFileVersion(version, 16);

    // バージョン番号をセット
    SetWindowText(GetDlgItem(hWnd, IDC_STATIC_PRODUCT), version);

    // ウィンドウを中央位置へ移動
    CenterWindow(hWnd);

    return FALSE;
}


//-----------------------------------------------------------------------------
// @outline     進捗ダイアログのコールバック関数です。
// @explanation 進捗ダイアログのメッセージを処理します。
// @param       [IN] hWnd   ダイアログボックスのハンドルを指定します。
//              [IN] msg    メッセージを指定します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return      メッセージを処理した場合には TRUE 以外を、処理しなかった場合には FALSE を返します。
//-----------------------------------------------------------------------------
BOOL CALLBACK ProgressDlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    HWND    hWndParent      = NULL; // 親ウィンドウのウィンドウハンドル
    TCHAR   szLabel[256];           // ラベル表示文字列
    TCHAR   szBuff[256];

    switch (msg) {
        // ダイアログの初期化
        case WM_INITDIALOG:
            // ウィンドウ位置を中央へ移動
            return CenterWindow(hWnd);

        // 進捗率更新
        case WM_USER_UPDATE_PROGRESS:
            // ステップ別の表示ラベルをセット
            switch ((INT)wParam) {
                // ファイルの読みこみ
                case SW_STEP_FILE_READ:
                    LoadString(g_hInstance, IDS_STEP_FILE_READ, szBuff, sizeof(szLabel));
                    break;

                // プログラミング・モードへの遷移
                case SW_STEP_MODE_CHANGE:
                    LoadString(g_hInstance, IDS_STEP_MODE_CHANGE, szBuff, sizeof(szLabel));
                    break;

                // ブロック消去
                case SW_STEP_BLOCK_ERASE:
                    LoadString(g_hInstance, IDS_STEP_BLOCK_ERASE, szBuff, sizeof(szLabel));
                    break;

                // プログラミング
                case SW_STEP_PROGRAMMING:
                    LoadString(g_hInstance, IDS_STEP_PROGRAMMING, szBuff, sizeof(szLabel));
                    break;

                // EEPROM書き換え
                case SW_STEP_CHG_EEPROM:
                    LoadString(g_hInstance, IDS_STEP_CHG_EEPROM, szBuff, sizeof(szLabel));
                    break;

                default:
                    wsprintf(szLabel, _T(""));
            }
            // ラベルの書式化
            wsprintf(szLabel, _T("%s (%d %%)"), szBuff, (UINT)lParam);

            // ラベル更新
            SetWindowText(GetDlgItem(hWnd, IDC_STATIC_LABEL), szLabel);
            
            // 進捗率更新
            SendMessage(GetDlgItem(hWnd, IDC_PROGRESS1), PBM_SETPOS, (UINT)lParam, 0);

            break;

        // クローズ
        case WM_CLOSE:

			// 親ウィンドウのウィンドウハンドルを取得する。
            hWndParent = GetParent(hWnd);

            if (NULL != hWndParent) {
                // 呼び出し元のウィンドウを有効化する。
                EnableWindow(hWndParent, TRUE);

                // 呼び出し元のウィンドウにフォーカスを当てる
                SetFocus(hWndParent);
            }

            // 自身を破棄する。
            DestroyWindow(g_hWndProgressDlg);
            break;
    }

    return FALSE;
}


//-----------------------------------------------------------------------------
// @outline     ファイルエディットのサブクラス化されたコールバック関数です。
// @explanation ファイルエディット(IDC_EDIT_DROP)のメッセージを処理します。
// @param       [IN] hWnd   ダイアログボックスのハンドルを指定します。
//              [IN] msg    メッセージを指定します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return      メッセージを処理した場合には TRUE 以外を、処理しなかった場合には FALSE を返します。
//-----------------------------------------------------------------------------
LRESULT CALLBACK EditDropSubWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
        // ドロップファイル
	    case WM_DROPFILES:
            HDROP   hDropInfo;                  // ファイル名構造体のハンドル
            TCHAR   szName      [MAX_PATH];     // ドロップされたファイルのフルパス
            TCHAR   szDrv       [MAX_PATH];     // ドライブ名
            TCHAR   szDir       [MAX_PATH];     // ディレクトリ名
            TCHAR   szFileName  [MAX_PATH];     // ファイル名
            TCHAR   szExtension [MAX_PATH];     // 拡張子名
            UINT	nFileCount;                 // ドロップされたファイル数
            UINT	nSize;	                    // ドロップされたファイル名の長さ

            // ファイル名構造体のハンドルを取得
            hDropInfo = (HDROP)wParam;
 
            // ドロップされたファイル数を取得
            nFileCount = DragQueryFile(hDropInfo, 0xFFFFFFFF, NULL, 0);
 
            //	ドロップされたファイルが1個の場合のみ受け付ける。
            if(	nFileCount == 1 ) {
                // ドロップされたファイル名の長さを取得
                nSize = DragQueryFile(hDropInfo, 0, NULL, 0);
                // ドロップされたファイルのフルパスを取得
                DragQueryFile(hDropInfo, 0, szName, nSize + 1);

                // ファイルパスを分解
                _tsplitpath_s(  szName, 
                                szDrv, 
                                szDir, 
                                szFileName,
                                szExtension);

                // 拡張子判定(小文字)
                if (0 == _tcsicmp(_T(".Bin"), szExtension) ||
					0 == _tcsicmp(_T(".Mot"), szExtension) ) {
                    // 最初に拡張子が一致したファイル名を設定
                    SetWindowText(hWnd, szName);
                    // 継続処理中断
                    break;
                }
            }
            
		    return FALSE;
	}

	// 未処理のメッセージの処理
	return CallWindowProc(g_wndProcEditDrop, hWnd, msg, wParam, lParam);
}


//-----------------------------------------------------------------------------
// @outline     F/W更新ログエディットのサブクラス化されたコールバック関数です。
// @explanation F/W更新ログエディット(IDC_EDIT_UPDATE_LOG)のメッセージを処理します。
// @param       [IN] hWnd   ダイアログボックスのハンドルを指定します。
//              [IN] msg    メッセージを指定します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return      メッセージを処理した場合には TRUE 以外を、処理しなかった場合には FALSE を返します。
//-----------------------------------------------------------------------------
LRESULT CALLBACK EditUpdateLogSubWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	// ウィンドウが閉じている時はメッセージを送付しない
	if( g_wndProcEditUpdateLog == NULL ) {
		return TRUE;
	}

    switch (msg) {
        // F/W更新ログクリア
        case WM_USER_INIT_UPDATE_LOG:
            SetWindowText(hWnd, _T(""));
            return TRUE;
        default:
            break;
    }
    // 未処理のメッセージの処理
    return CallWindowProc(g_wndProcEditUpdateLog, hWnd, msg, wParam, lParam);
}


//-----------------------------------------------------------------------------
// @outline     メッセージボックス表示前のフックプロシージャ関数です。
// @explanation メッセージボックス表示前のフックプロシージャ関数です。
// @param       [IN] code   フックタイプを指定します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return      CBTフックコードに対応する操作を認める場合は 0を、拒否する場合は 1 を返します。
//-----------------------------------------------------------------------------
LRESULT CALLBACK CBTActivateMessageBox(int code, WPARAM wParam, LPARAM lParam)
{
    // システムがウィンドウをアクティベート以外は無視
	if (HCBT_ACTIVATE != code /*|| -1 == code*/)	{
		// 次のフックプロシージャを呼び出す。
		return CallNextHookEx(hCBTHook, code, wParam, lParam);
	}
	
	HWND hWnd = (HWND)wParam;
    
    // フックチェーン内にインストールされたフックプロシージャを削除
	UnhookWindowsHookEx(hCBTHook);

    // ウィンドウを中央へ移動する。
	CenterWindow(hWnd);

    // 次のフックプロシージャを呼び出す。
	return CallNextHookEx(hCBTHook, code, wParam, lParam);
}


//-----------------------------------------------------------------------------
// @outline     ファイルのバージョン番号を取得します。
// @explanation ファイルのバージョン番号を取得します。
// @param       [OUT] *version  バージョン番号を格納する文字列のポインタを指定しています。
//              [IN]  iLen      バージョン番号を格納する文字列のバッファサイズを指定します。
// @return      none
//-----------------------------------------------------------------------------
void GetFileVersion(TCHAR *version, INT iLen)
{
    DWORD               size, dw;
    TCHAR               path[_MAX_PATH + 1];
    LPVOID              lp, lpbuf;
    UINT	            usize;
    VS_FIXEDFILEINFO*   info;
    WORD                ver[4];
        
    if (GetModuleFileName(NULL, path, _MAX_PATH) != 0) {
        if ((size = GetFileVersionInfoSize(path, &dw)) > 0) {
            if ((lp = GlobalAlloc(GMEM_FIXED, size)) != NULL) {
                GetFileVersionInfo(path, 0, size, lp);
                VerQueryValue(lp, _T(""), &lpbuf, &usize);
                info = (VS_FIXEDFILEINFO*)lpbuf;
                ver[3] = HIWORD(info->dwFileVersionMS);
                ver[2] = LOWORD(info->dwFileVersionMS);
                ver[1] = HIWORD(info->dwFileVersionLS);
                ver[0] = LOWORD(info->dwFileVersionLS);
                // MEMO:バージョン表記の4桁目は将来の拡張用とし、
                //      
                wsprintf(version, _T("V%d.%02d"), ver[3], ver[2]);
                GlobalFree(lp);
            }
        }
    }
}


//-----------------------------------------------------------------------------
// @outline		シリアルポート情報の初期化を行います。
// @explanation	シリアルポート情報を初期化してコンボボックスにアイテムを設定します。
// @param		[IN] hWnd   呼び出し元のウィンドウハンドルを指定します。
// @return		none
//-----------------------------------------------------------------------------
void InitializeSerialPortInfo(HWND hWnd)
{
    DWORD           dwErr = SW_SUCCESS;
    wstringstream   buff;

    // アイテムの全削除
    SendMessage(GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT), CB_RESETCONTENT, 0, 0);
    g_SerialPortInfoList.clear();

    // システム上のシリアルポート情報を取得
    dwErr = CSerialPortProcessor::GetSerialPortInfo(&g_SerialPortInfoList);

	if (!dwErr) {
        // コンボボックスにアイテムを追加
        for (UINT i = 0; i < g_SerialPortInfoList.size(); i++) {
            SERIALPORTINFO info = g_SerialPortInfoList.at(i);
            buff << info.portName << _T(" : ") << info.description;
            SendMessage(GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT), CB_INSERTSTRING, i, (LPARAM)buff.str().data());
            SendMessage(GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT), CB_SETITEMDATA, i, (LPARAM)info.portNumber);
            buff.str(_T(""));
        }

        SendMessage(GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT), CB_SETCURSEL, g_SerialportSelectedIndex, 0);   
    }
}


//-----------------------------------------------------------------------------
// @outline		シリアルポート選択コンボボックスのドロップダウンメッセージを処理します。
// @explanation	シリアルポート選択コンボボックスのドロップダウン時の項目名の表示列幅を調整します。
// @param		[IN] hWnd   呼び出し元のウィンドウハンドルを指定します。
// @return		none
//-----------------------------------------------------------------------------
void OnCbnDropdownComboSerialPoprt(HWND hWnd)
{
    int			nElementCount	= 0;                                // コンボ・ボックスの要素数
    int			nScrollWidth	= ::GetSystemMetrics(SM_CXVSCROLL); // 垂直スクロール・バーの幅
	int			nWidth			= 0;                                // ドロップ・ダウンリストの幅
    HWND        hWndCbo         = NULL;
    SIZE        buffSize;

    // シリアルポート選択コンボボックスのウィンドウハンドルを取得
    hWndCbo = GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT);

    // コンボボックスの要素数取得
    nElementCount = SendMessage(hWndCbo, CB_GETCOUNT, 0, 0);
	
    // コンボ・ボックスのデバイス・コンテキストを取得
    HDC hDc = GetDC(hWndCbo);
	
    // フォントを選択
    SelectObject(hDc, (HFONT)SendMessage(hWndCbo, WM_GETFONT, 0, 0));

    TCHAR   szBuffStr[MAX_PATH];
    // コンボ・ボックスの要素数分
	for (int i = 0; i < nElementCount; i++) {
        // 要素[i]の文字列を取得
        SendMessage(hWndCbo, CB_GETLBTEXT, i, (LPARAM)szBuffStr);
        int textLen = SendMessage(hWndCbo, CB_GETLBTEXTLEN, i, 0);
        // 要素[i]の幅を取得
        GetTextExtentPoint32(hDc, szBuffStr, textLen, &buffSize);
        int nLength = buffSize.cx + nScrollWidth;
        // 最大幅の更新
		nWidth = max(nWidth, nLength);
    }
    
	// マージン設定
    GetTextExtentPoint32(hDc, _T("0"), 1, &buffSize);
    nWidth += buffSize.cx;

    // デバイスコンテキスト解放
    ReleaseDC(hWndCbo, hDc);
    
	// ドロップダウン時の表示幅を設定
    SendMessage(hWndCbo, CB_SETDROPPEDWIDTH, nWidth, 0);
}


//-----------------------------------------------------------------------------
// @outline		シリアルポート選択コンボボックスの選択アイテム変更メッセージを処理します。
// @explanation	シリアルポート選択コンボボックスの選択アイテム変更メッセージを処理します。
// @param		[IN] hWnd   呼び出し元のウィンドウハンドルを指定します。
// @return		none
//-----------------------------------------------------------------------------
void OnCbnSelchangeComboSerialPoprt(HWND hWnd)
{
    // 選択されたアイテムのインデックスをグローバル変数へ保存
    g_SerialportSelectedIndex = SendMessage(GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT), CB_GETCURSEL, 0, 0);
}


//-----------------------------------------------------------------------------
// @outline		ボーレート選択コンボボックスの初期化を行います。
// @explanation	ボーレート選択コンボボックスの初期化を行います。
// @param		none
// @return		none
//-----------------------------------------------------------------------------
void InitializeBaudRate(HWND hWnd)
{
    // コンボボックスのアイテムを削除
    SendMessage(GetDlgItem(hWnd, IDC_COMBO_BAUD_RATE), CB_RESETCONTENT, 0, 0);

    // コンボボックスのアイテムリストを追加
    SendMessage(GetDlgItem(hWnd, IDC_COMBO_BAUD_RATE), CB_INSERTSTRING, 0, (LPARAM)_T("115,200 bps"));
    SendMessage(GetDlgItem(hWnd, IDC_COMBO_BAUD_RATE), CB_SETITEMDATA, 0 ,(DWORD)BPS_115200);

    SendMessage(GetDlgItem(hWnd, IDC_COMBO_BAUD_RATE), CB_INSERTSTRING, 1, (LPARAM)_T("500,000 bps"));
    SendMessage(GetDlgItem(hWnd, IDC_COMBO_BAUD_RATE), CB_SETITEMDATA, 1 ,(DWORD)BPS_500000);
    
    
    // 最初の選択肢を選択する。
    SendMessage(GetDlgItem(hWnd, IDC_COMBO_BAUD_RATE), CB_SETCURSEL, 0, 0);
}

//-----------------------------------------------------------------------------
// @outline		Monitoring用ボーレート選択コンボボックスの初期化を行います。
// @explanation	Monitoring用ボーレート選択コンボボックスの初期化を行います。
// @param		none
// @return		none
//-----------------------------------------------------------------------------
void InitializeMonitorBaudRate(HWND hWnd)
{
    // コンボボックスのアイテムを削除
    SendMessage(GetDlgItem(hWnd, IDC_COMBO_MONITOR_BAUD_RATE), CB_RESETCONTENT, 0, 0);

    // コンボボックスのアイテムリストを追加
	DWORD	bps[] = {4800, 9600, 19200, 57600, 115200, 230400, 460800, 921600 };
	TCHAR	strBps[256];

	for( int i = 0; i < 8; i ++)
	{
		swprintf_s( strBps, 256, _T("%d"), bps[i]);
		SendMessage(GetDlgItem(hWnd, IDC_COMBO_MONITOR_BAUD_RATE), CB_INSERTSTRING, i, (LPARAM)strBps);
		SendMessage(GetDlgItem(hWnd, IDC_COMBO_MONITOR_BAUD_RATE), CB_SETITEMDATA, i ,bps[i]);
	}   
    
    // 最初の選択肢を選択する。
    SendMessage(GetDlgItem(hWnd, IDC_COMBO_MONITOR_BAUD_RATE), CB_SETCURSEL, 1, 0);
}

//-----------------------------------------------------------------------------
// @outline		ファイル選択ボタンのクリックメッセージを処理します。
// @explanation	ファイル選択ボタンのクリックメッセージを処理します。
// @param		[IN] hWnd   呼び出し元のウィンドウハンドルを指定します。
// @return		none
//-----------------------------------------------------------------------------
void OnBnClickedButtonFileSelect(HWND hWnd)
{
    static OPENFILENAME ofn;                        // OPENFILENAME構造体
    static TCHAR        szFullPathName[MAX_PATH];   // フルパスファイル名
	static TCHAR        szFilename[MAX_PATH];       // ファイル名
    static TCHAR        szFilter[MAX_PATH];         // フィルタ
    static TCHAR        szCaption[MAX_PATH];        // キャプション
    static TCHAR        szDefEXT[MAX_PATH];         // デフォルトのファイルの種類
    
    // フィルタを取得
    LoadString(g_hInstance, IDS_FILE_FILTER, szFilter, MAX_PATH);
    // キャプションを取得
    LoadString(g_hInstance, IDS_OPEN_FILE_CAPTION, szCaption, MAX_PATH);
    // デフォルトのファイルの種類を取得
    LoadString(g_hInstance, IDS_OPEN_FILE_DEF_EXT, szDefEXT, MAX_PATH);

    // 構造体に情報をセット
	ZeroMemory(&ofn, sizeof(ofn));
	ofn.lStructSize     = sizeof(ofn);                          // 構造体のサイズ
	ofn.hwndOwner       = hWnd;                                 // 親ウィンドウのハンドル
    ofn.lpstrFilter     = szFilter;                             // フィルタ
	ofn.lpstrFile       = szFullPathName;                       // 選択されたファイル名(フルパス)を受け取る変数のアドレス
	ofn.lpstrFileTitle  = szFilename;                           // 選択されたファイル名を受け取る変数のアドレス
	ofn.nMaxFile        = sizeof(szFullPathName);               // lpstrFileに指定した変数のサイズ
	ofn.nMaxFileTitle   = sizeof(szFilename);                   // lpstrFileTitleに指定した変数のサイズ
	ofn.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY; // フラグ指定
    ofn.lpstrTitle      = szCaption;                            // キャプション
	ofn.lpstrDefExt     = szDefEXT;                             // デフォルトのファイルの種類

	// ファイルを開くコモンダイアログを作成
    if (FALSE != GetOpenFileName(&ofn)) {
        SetWindowText(GetDlgItem(hWnd, IDC_EDIT_DROP), szFullPathName);
    }
}


//-----------------------------------------------------------------------------
// @outline		書き込みボタンのクリックメッセージを処理します。
// @explanation	書き込みボタンのクリックメッセージを処理します。
// @param		[IN] hWnd   呼び出し元のウィンドウハンドルを指定します。
// @return		none
//-----------------------------------------------------------------------------
void OnBnClickedButtonWrite(HWND hWnd)
{
    DWORD       dwRet           = SW_SUCCESS;   // 戻り値
    TCHAR       buff[260];

	TCHAR   szDrv       [MAX_PATH];     // ドライブ名
    TCHAR   szDir       [MAX_PATH];     // ディレクトリ名
    TCHAR   szFileName  [MAX_PATH];     // ファイル名
    TCHAR   szExtension [MAX_PATH];     // 拡張子名

	// シリアルモニタ用接続を解除する
	MonitorPortClose(hWnd);


    // 自ダイアログを無効化する。
    EnableWindow(hWnd, FALSE);
    
    // 自ダイアログの表示を更新する。
    UpdateWindow(hWnd);

    // コンテキスト構造体の初期化
    ZeroMemory(&g_ContextInfo, sizeof(CONTEXTINFO));
    memset(&g_ContextInfo.SiliconSignature.DevName, NULL, sizeof(g_ContextInfo.SiliconSignature.DevName));

    // デバイス変更構造体の初期化
    ZeroMemory(&g_DevChangeInfo, sizeof(DEVCHANGEINFO));
    g_DevChangeInfo.hNotify = INVALID_HANDLE_VALUE;
    g_ContextInfo.pDevInfo = &g_DevChangeInfo;

    // 実行結果情報を初期化
    ZeroMemory(&g_ResultInfo, sizeof(RESULTINFO));
    g_ContextInfo.pResultInfo = &g_ResultInfo;

    // 呼び出し元ウィンドウを設定
    g_ContextInfo.ParentWindow = hWnd;

    // 進捗ダイアログの生成と表示
    g_hWndProgressDlg = CreateDialog(g_hInstance, MAKEINTRESOURCE(IDD_DIALOG_STATUS), hWnd, ProgressDlgProc);
    ShowWindow(g_hWndProgressDlg, SW_SHOW );

    // 進捗情報の通知先を進捗ダイアログに設定
    g_ContextInfo.NotifyProgress = g_hWndProgressDlg;

    // F/W更新ログ情報の通知先をエディットコンソールに設定
    g_ContextInfo.UpdateLog = g_hEditUpdateLog;

    // DTR反転フラグを設定
    g_ContextInfo.IsInvertDtr = SendMessage(GetDlgItem(hWnd, IDC_CHECK_FTDI), BM_GETCHECK, 0, 0) == BST_CHECKED ? TRUE : FALSE;

    // 選択されているシリアルポートのポート番号を取得
    HWND hWndComboSerialPort = GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT);
    if (0UL < SendMessage(hWndComboSerialPort, CB_GETCOUNT, 0, 0)) {
        // 選択されているアイテムのインデックス取得
        int nSelectedIndex = SendMessage(hWndComboSerialPort, CB_GETCURSEL, 0, 0);
        if (CB_ERR != nSelectedIndex) {
            if (CB_ERR != SendMessage(hWndComboSerialPort, CB_GETLBTEXTLEN, nSelectedIndex, 0)) {
                g_ContextInfo.PortNumber = g_SerialPortInfoList[nSelectedIndex].portNumber;
                g_DevChangeInfo.PortNumber = g_SerialPortInfoList[nSelectedIndex].portNumber;
            }
            else {
                // 選択されていない場合
                dwRet = SW_ERROR_SERIAL_UNSELECT;
            }
        }
        else {
            // 選択されていない場合
            dwRet = SW_ERROR_SERIAL_UNSELECT;
        }
    }

    // 選択されているボーレートを取得
    if (SW_SUCCESS == dwRet) {
        HWND hWndComboBaudRate = GetDlgItem(hWnd, IDC_COMBO_BAUD_RATE);
        int nSelectedIndex = SendMessage(hWndComboBaudRate, CB_GETCURSEL, 0, 0);
        g_ContextInfo.BaudRate = 
            (BYTE)SendMessage(hWndComboBaudRate, CB_GETITEMDATA, nSelectedIndex, 0);
    }
    
    if (SW_SUCCESS == dwRet) {
        // 選択されているファイルのフルパスを取得
        GetWindowText(GetDlgItem(hWnd, IDC_EDIT_DROP), g_ContextInfo.TargetFileInfo.FilePath, MAX_PATH);

		// ファイルの拡張子に応じて書き込み処理を分ける
		_tsplitpath_s(g_ContextInfo.TargetFileInfo.FilePath,
			szDrv, 
			szDir, 
			szFileName,
			szExtension);

		if (0 == _tcsicmp(_T(".mot"), szExtension)) {
			// Motファイル用：FW書き換え処理を開始する。
			dwRet = g_SequenceProcessorForMot.Start(g_ContextInfo);
		}else {
			// binファイル用：FW書き換え処理を開始する。
			dwRet = g_SequenceProcessor.Start(g_ContextInfo);
		}

        
    }

    if (SW_SUCCESS != dwRet) {
        // 開始できない場合
        // 進捗ダイアログを閉じる。
        SendMessage(g_ContextInfo.NotifyProgress, WM_SYSCOMMAND, SC_CLOSE, 0);

        // 自ダイアログを有効化する。
        EnableWindow(hWnd, TRUE);

        // ユーザへ通知
        TCHAR   szMessage[256];
        UINT    uType;

        LoadString(g_hInstance, IDS_WRITE_FAIL, buff, sizeof(buff));
        wsprintf(szMessage, _T("%s\r\nERROR:0x%08X\r\n"), buff, dwRet);
        uType = MB_ICONERROR | MB_OK;

        MessageBox(hWnd, szMessage, NULL, uType);
    }
}

//-----------------------------------------------------------------------------
// @outline		モニタ用シリアルポートオープンボタンのクリックメッセージを処理します。
// @explanation	モニタ用シリアルポートオープンボタンのクリックメッセージを処理します。
// @param		[IN] hWnd   呼び出し元のウィンドウハンドルを指定します。
// @return		none
//-----------------------------------------------------------------------------
void OnBnClickedButtonMonitorOpen(HWND hWnd)
{
	// 現在の状態を確認
	if( g_isMonitorOpen == false ) {
		// 未接続状態の場合：オープン処理
		
		g_isInvertDTR = false;

		// DTRの状態確認
		DtrCheck(hWnd);

		// ポート接続
		MonitorPortOpen(hWnd);
	}
	else {
		// 接続済み状態の場合：クローズ処理
		MonitorPortClose(hWnd);
	}
}

//-----------------------------------------------------------------------------
// @outline		実行結果メッセージを処理します。
// @explanation	実行結果メッセージを処理します。
// @param		[IN] hWnd   呼び出し元のウィンドウハンドルを指定します。
// @return		none
//-----------------------------------------------------------------------------
void OnResult(HWND hWnd)
{
    DWORD   dwRet       = SW_SUCCESS;
    TCHAR   buff[1024];
    wstring message;
    UINT    uType;

    if (SW_SUCCESS == g_ContextInfo.pResultInfo->Result) {
        LoadString(g_hInstance, IDS_WRITE_SUCCESS, buff, sizeof(buff));
        message = buff;
        uType = MB_ICONINFORMATION | MB_OK;
    }
    else {
        LoadString(g_hInstance, IDS_WRITE_FAIL, buff, sizeof(buff));
        message = buff;
        wsprintf(buff, _T("\r\nERROR:0x%08X\r\n"), g_ContextInfo.pResultInfo->Result);
        message += buff;
        wsprintf(buff, _T("COM = 0x%02X, Retry = %u\r\nST1 = 0x%02X, DST1 = 0x%02X, DST2 = 0x%02X\r\n"),
                             g_ContextInfo.pResultInfo->ComInfo.Command,
                             g_ContextInfo.pResultInfo->ComInfo.Retry != 0 ? g_ContextInfo.pResultInfo->ComInfo.Retry + 1 : 0,
                             g_ContextInfo.pResultInfo->ComInfo.ST1,
                             g_ContextInfo.pResultInfo->ComInfo.DST1,
                             g_ContextInfo.pResultInfo->ComInfo.DST2);
        message += buff;
        uType = MB_ICONERROR | MB_OK;
    }

    // MEMO:Win32APIのメッセージボックスは常に画面の中央で表示されるため、
    //      フックを使って表示する前に表示位置を親ウィンドウの中央へ移動する。
    // フックプロシージャをフックチェーン内にインストール
    hCBTHook = SetWindowsHookEx(WH_CBT, CBTActivateMessageBox, NULL, ::GetCurrentThreadId());
    
    // メッセージボックスを表示
    MessageBox(hWnd, message.c_str(), _T(""), uType);
    
    // フックチェーン内にインストールされたフックプロシージャを削除
    UnhookWindowsHookEx(hCBTHook);
}


//-----------------------------------------------------------------------------
// @outline		デバイス変更通知のメッセージを処理します。
// @explanation	デバイス変更通知のメッセージを処理します。
//              [IN] wParam 最初のメッセージパラメータを指定します。
//              [IN] lParam 2番目のメッセージパラメータを指定します。
// @return		none
//-----------------------------------------------------------------------------
void OnDeviceChange(WPARAM wParam, LPARAM lParam)
{
    if (TRUE == g_DevChangeInfo.IsDeviceChange) {
	    switch(wParam) {
            // デバイスが挿入されていて使用できる状態の場合
	        case DBT_DEVICEARRIVAL:
                // デバイスのブロードキャストヘッダを取得
                DEV_BROADCAST_HDR* pHdr = (DEV_BROADCAST_HDR*)lParam;
        
                switch(pHdr->dbch_devicetype) {
                    // ポートデバイスの場合
                    case DBT_DEVTYP_PORT: 
                        // ポートデバイスのブロードキャストヘッダを取得
                        DEV_BROADCAST_PORT *pPort = (DEV_BROADCAST_PORT*)pHdr;
                        if (_tcsncicmp(pPort->dbcp_name, _T("COM"), 3) == 0) {
                            if (g_DevChangeInfo.PortNumber == _tstoi(&(pPort->dbcp_name[3]))) {
                                DTRACE(_T("対象デバイスが使用可能"));
                                if (INVALID_HANDLE_VALUE != g_DevChangeInfo.hNotify) {
                                    g_DevChangeInfo.IsDeviceChange = FALSE;
                                    // イベント通知
                                    SetEvent(g_DevChangeInfo.hNotify);
                                }
                            }
                            
                        }
                        break;
                    }
                    break;
        }
    }
}

//-----------------------------------------------------------------------------
// @outline		ツールヒント用 TTN_NEEDTEXT 通知のメッセージを処理します。
// @explanation	コントロールのツールヒント用 TTN_NEEDTEXT 通知を処理してツールヒントを設定します。
//				※TTN_NEEDTEXT 通知ではpResultの値は無視されるため設定不要
// @param		[IN]pNMHDR	NMTTDISPINFO 構造体へのポインタを指定します。
//				[IN]pResult	結果コードへのポインタを指定います。
// @return		void
//-----------------------------------------------------------------------------
void OnNeedText(HWND hWnd, NMHDR* pNMHDR)
{
    // ツールチップテキスト構造体を取得
	TOOLTIPTEXT* pTTT = (TOOLTIPTEXT*)pNMHDR;

    // ツールへのウィンドウ・ハンドルを含むかどうか
    if(pTTT->uFlags & TTF_IDISHWND){
        TCHAR buff[MAX_PATH];
        // 通知するコントロールのIDを取得
        int nID = ::GetDlgCtrlID((HWND)pTTT->hdr.idFrom);
        switch (nID) {
        case IDC_COMBO_SERIAL_POPRT:
            // 現在の選択アイテムの文字列長が0を超える場合
            if (0 < SendMessage(GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT), CB_GETLBTEXTLEN, g_SerialportSelectedIndex, 0)) {
                // 選択アイテムの文字列を取得
                SendMessage(GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT), CB_GETLBTEXT, g_SerialportSelectedIndex, (LPARAM)buff);
                // ツールチップへセット
                lstrcpy(pTTT->szText, buff);
            }
            break;
        }
    }
}


//-----------------------------------------------------------------------------
// @outline     ウィンドウを画面または親ウィンドウの中央へ移動します。
// @explanation 親ウィンドウが存在する場合には親ウィンドウの中央へ、存在しない場合には画面中央へ移動します。
// @param       [IN] hWnd   移動するウィンドウのハンドルを指定します。
// @return      移動できた場合には 0 以外を、移動できなかった場合には 0 を返します。
//-----------------------------------------------------------------------------
BOOL CenterWindow(HWND hWnd)
{
    RECT    rc1;        // デスクトップまたは親ウィンドウ領域
    RECT    rc2;        // ウインドウ領域
    INT     cx, cy;     // ウインドウ位置
    INT     sx, sy;     // ウインドウサイズ
    HWND    hParentWnd; // 親ウィンドウのハンドル

    // 親ウィンドウのハンドルを取得する。
    hParentWnd = NULL;
    hParentWnd = GetParent(hWnd);
    
    // サイズの取得
    if (NULL == hParentWnd) {
        // 親ウィンドウが存在しない場合
        GetMonitorRect(&rc1);                               // デスクトップのサイズ
    }
    else {
        // 親ウィンドウが存在する場合
        GetWindowRect(hParentWnd, &rc1);                    // 親ウィンドウのサイズ
    }
    GetWindowRect(hWnd, &rc2);                              // ウインドウのサイズ
    
    sx = (rc2.right - rc2.left);                            // ウインドウの横幅
    sy = (rc2.bottom - rc2.top);                            // ウインドウの高さ
    cx = (((rc1.right - rc1.left) - sx) / 2 + rc1.left);    // 横方向の中央座標軸
    cy = (((rc1.bottom - rc1.top) - sy) / 2 + rc1.top);     // 縦方向の中央座標軸
    
    // 中央に移動
    return SetWindowPos(hWnd, NULL, cx, cy, 0, 0, (SWP_NOZORDER | SWP_NOOWNERZORDER | SWP_NOSIZE));
}

//-----------------------------------------------------------------------------
// @outline     シリアルモニタ用　ポート切断
// @explanation シリアルモニタ用にオープンしているポートを切断します。コントロールも復旧します。
// @param       [IN] hWnd   対象ウィンドウのハンドルを指定します。
// @return      void
//-----------------------------------------------------------------------------
void MonitorPortClose(HWND hWnd)
{

	// シリアルモニタが接続中の場合は、DTRの解除処理
	if( true == g_isMonitorOpen ) {
		if( false == g_isInvertDTR ) {
			g_MonitorComPort.ClearDtr();
		}
		else {
			g_MonitorComPort.SetDtr();
		}
	}

	g_isMonitorOpen = false;
	
	// タイマの停止
	MonitorTimerStop(hWnd);
	
	// ポートの停止
	g_MonitorComPort.Close();

	// コントロールを有効化
	SetControlEnabledModeMonitor( hWnd, true);
}

//-----------------------------------------------------------------------------
// @outline     シリアルモニタ用　ポートオープン
// @explanation シリアルモニタ用にシリアルポートをオープンします。
// @param       [IN] hWnd   対象ウィンドウのハンドルを指定します。
// @return      処理結果
//-----------------------------------------------------------------------------
DWORD MonitorPortOpen(HWND hWnd)
{
	DWORD	dwRet = SW_SUCCESS;
	TCHAR	baudRate[256];
	DCB		dcb;                        // DCB構造体
	DWORD	setBaudRate;
	int		portNumber;

	TCHAR   szMessage[256];
	TCHAR	buff[260];
    UINT    uType;

	// 最初に切断処理を行う
	MonitorPortClose(hWnd);

	// ボーレートの読込処理
	HWND hWndComboBaudRate = GetDlgItem(hWnd, IDC_COMBO_MONITOR_BAUD_RATE);
    GetWindowTextW( hWndComboBaudRate, (LPWSTR)baudRate, 256);
	try {
		setBaudRate = StrToIntW(baudRate);
	}
	catch(...) {
		setBaudRate = 9600;
		dwRet = SW_ERROR_CHG_BAUDRATE;
	}	

	// ボーレートの上下限値チェック
	if( MIN_BAUDRATE > setBaudRate ||
		MAX_BAUDRATE < setBaudRate ){
		setBaudRate = 9600;
		dwRet = SW_ERROR_CHG_BAUDRATE;
	}
	
	if( dwRet == SW_SUCCESS ) {
		// シリアルポートの設定	
		memset(&dcb, 0, sizeof(DCB));               // DCB構造体の初期化
		dcb.DCBlength       = sizeof(DCB) ;         // DCB構造体のサイズ
		dcb.fBinary         = TRUE;                 // Windowsはバイナリモードのみサポート
		dcb.BaudRate	    = CBR_115200;			// ボーレート		：入力された値
		dcb.Parity		    = NOPARITY;				// パリティビット	：なし
		dcb.ByteSize	    = 8;					// データ長			：8ビット
		dcb.StopBits	    = ONESTOPBIT;			// ストップビット	：1ビット
		dcb.fDtrControl     = DTR_CONTROL_DISABLE;  // DTR制御線を無効化

		// 選択されているシリアルポートのポート番号を取得
		HWND hWndComboSerialPort = GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT);
		if (0UL < SendMessage(hWndComboSerialPort, CB_GETCOUNT, 0, 0)) {
			// 選択されているアイテムのインデックス取得
			int nSelectedIndex = SendMessage(hWndComboSerialPort, CB_GETCURSEL, 0, 0);
			if (CB_ERR != nSelectedIndex) {
				if (CB_ERR != SendMessage(hWndComboSerialPort, CB_GETLBTEXTLEN, nSelectedIndex, 0)) {
					portNumber = g_SerialPortInfoList[nSelectedIndex].portNumber;
				}
				else {
					// 選択されていない場合
					dwRet = SW_ERROR_SERIAL_UNSELECT;
				}
			}
			else {
				// 選択されていない場合
				dwRet = SW_ERROR_SERIAL_UNSELECT;
			}
		}
	}

	// シリアルポートの接続
	if( dwRet == SW_SUCCESS ){
		// 受信バッファのクリア
	    g_MonitorComPort.ClearReceiveBuffer();

        dwRet = g_MonitorComPort.Open(portNumber, &dcb);
        if (SW_SUCCESS == dwRet) {
            // 受信スレッドの起動待ち
            while (FALSE == g_MonitorComPort.IsReading()) {
                Sleep(0);
            }
        }	
	}

	// ボーレートの変更
	if( SW_SUCCESS == dwRet ){
		dwRet = g_MonitorComPort.ChangeBaudRate(setBaudRate);
	}

	// SetDTRコマンド発行
	if( dwRet == SW_SUCCESS ){
		if( false == g_isInvertDTR ) {
			dwRet = g_MonitorComPort.SetDtr();
		}
		else {
			dwRet = g_MonitorComPort.ClearDtr();
		}
	}	

	// タイマの作成
	if( dwRet == SW_SUCCESS ){
		// タイマの作成
		UINT ans = MonitorTimerStart(hWnd);
		if( ans <= 0 ) {
			dwRet = SW_ERROR_MONITOR_TIMER_INIT;
		}
	}

	if( dwRet == SW_SUCCESS ){

		// F/W更新ログのクリア
	    SendMessage( g_hEditUpdateLog, WM_USER_INIT_UPDATE_LOG, 0, 0);

		// コントロールを無効化
		SetControlEnabledModeMonitor( hWnd, false);
		g_isMonitorOpen = true;
	}
	else{
		// エラー発生時はポートを閉じる
		MonitorPortClose(hWnd);
		
		// ユーザへ通知
		LoadString(g_hInstance, IDS_WRITE_FAIL, buff, sizeof(buff));
		wsprintf(szMessage, _T("%s\r\nERROR:0x%08X\r\n"), buff, dwRet);
		uType = MB_ICONERROR | MB_OK;
		MessageBox(hWnd, szMessage, NULL, uType);
	}

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline     DTRの状態確認
// @explanation ターゲットをプログラミングモードへ移行させることで、DTRの状態を取得します。
// @param       [IN] hWnd   対象ウィンドウのハンドルを指定します。
// @return      処理結果
//-----------------------------------------------------------------------------
DWORD DtrCheck(HWND hWnd)
{
	DWORD	dwRet = SW_SUCCESS;
	DCB		dcb;                        // DCB構造体
	int		portNumber;

	// 最初に切断処理を行う
	MonitorPortClose(hWnd);
	
	// シリアルポートの設定	
	memset(&dcb, 0, sizeof(DCB));               // DCB構造体の初期化
	dcb.DCBlength       = sizeof(DCB) ;         // DCB構造体のサイズ
	dcb.fBinary         = TRUE;                 // Windowsはバイナリモードのみサポート
	dcb.BaudRate	    = CBR_115200;			// ボーレート		：入力された値
	dcb.Parity		    = NOPARITY;				// パリティビット	：なし
	dcb.ByteSize	    = 8;					// データ長			：8ビット
	dcb.StopBits	    = ONESTOPBIT;			// ストップビット	：1ビット
	dcb.fDtrControl     = DTR_CONTROL_DISABLE;  // DTR制御線を無効化

	// 選択されているシリアルポートのポート番号を取得
	HWND hWndComboSerialPort = GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT);
	if (0UL < SendMessage(hWndComboSerialPort, CB_GETCOUNT, 0, 0)) {

		// 選択されているアイテムのインデックス取得
		int nSelectedIndex = SendMessage(hWndComboSerialPort, CB_GETCURSEL, 0, 0);
		if (CB_ERR != nSelectedIndex) {
			if (CB_ERR != SendMessage(hWndComboSerialPort, CB_GETLBTEXTLEN, nSelectedIndex, 0)) {
				portNumber = g_SerialPortInfoList[nSelectedIndex].portNumber;
			}
			else {
				// 選択されていない場合
				dwRet = SW_ERROR_SERIAL_UNSELECT;
			}
		}
		else {
			// 選択されていない場合
			dwRet = SW_ERROR_SERIAL_UNSELECT;
		}
	}

	// シリアルポートの接続
	if( dwRet == SW_SUCCESS ){
        dwRet = g_MonitorComPort.Open(portNumber, &dcb);
        if (SW_SUCCESS == dwRet) {
            // 受信スレッドの起動待ち
            while (FALSE == g_MonitorComPort.IsReading()) {
                Sleep(0);
            }
        }
	}
	
	// DTR確認処理
	if( SW_SUCCESS == dwRet ){
		g_isMonitorOpen = true;
		dwRet = ComBaudRateSetProc(0, V33); 
	}

	// プログラムモード移行時の状態チェック
	if( SW_SUCCESS != dwRet ) {
		g_isInvertDTR = false;		
	}
	else{
		g_isInvertDTR = true;
	}
	
	// ポートを切断する
	MonitorPortClose(hWnd);

	return dwRet;
}

//-----------------------------------------------------------------------------
// @outline     コントロールの有効/無効設定（シリアルモニタモード)
// @explanation シリアルモニタモード中のコントロールの有効/無効を切り替えます。
// @param       [IN] hWnd   対象ウィンドウのハンドルを指定します。
//				[IN] isEnabled	有効=true/無効=false
// @return      void
//-----------------------------------------------------------------------------
void SetControlEnabledModeMonitor(HWND hWnd, bool isEnabled)
{
	// 各コントロールの状態設定
	HWND hWndButtonWrite	= GetDlgItem(hWnd, IDC_BUTTON_WRITE);
	HWND hWndButtonFileSelect	= GetDlgItem(hWnd, IDC_BUTTON_FILE_SELECT);
	HWND hWndButtonUpdatePort = GetDlgItem(hWnd, IDC_BUTTON_UPDATE_SERIALPORT);

	HWND hWndCheckFtdi	= GetDlgItem(hWnd, IDC_CHECK_FTDI);
	HWND hWndComboBaudRate = GetDlgItem(hWnd, IDC_COMBO_BAUD_RATE);
	HWND hWndEditDrop = GetDlgItem(hWnd, IDC_EDIT_DROP);
	
	HWND hWndComboSerialPort = GetDlgItem(hWnd, IDC_COMBO_SERIAL_POPRT);	
	HWND hWndComboMonitorBaudRate = GetDlgItem(hWnd, IDC_COMBO_MONITOR_BAUD_RATE);	

	EnableWindow(hWndComboBaudRate, isEnabled);
	EnableWindow(hWndCheckFtdi, isEnabled);
	EnableWindow(hWndButtonWrite, isEnabled);
	EnableWindow(hWndButtonUpdatePort, isEnabled);
	EnableWindow(hWndComboSerialPort, isEnabled);
	EnableWindow(hWndComboMonitorBaudRate, isEnabled);
	EnableWindow(hWndEditDrop, isEnabled);
	EnableWindow(hWndButtonFileSelect, isEnabled);
	
	// Openボタンの状態切り替え
	TCHAR	label[256];
	if( isEnabled == true ) {
		wcscpy_s(label, 256, _T("Open"));		
	}
	else {
		wcscpy_s(label, 256, _T("Close"));		
	}
	SetDlgItemText( hWnd, IDC_BUTTON_MONITOR_OPEN, label);
}

//-----------------------------------------------------------------------------
// @outline     シリアルモニタ用タイマイベント処理
// @explanation シリアルモニタ用のメッセージ表示用処理を行います。
// @param       [IN] hWnd		イベントが発生したウィンドウのハンドルが入ります。
//				[IN] uMsg		受信したメッセージが入ります。
//				[IN] idEvent	タイマーの ID が入ります。
//				[IN] dwTime		Windows が起動してからの経過時間がミリ秒単位で入ります。
// @return      void
//-----------------------------------------------------------------------------
VOID CALLBACK MonitorTimerProc(HWND hWnd, UINT uMsg, UINT idEvent, DWORD dwTime) {

	// タイマを停止します
	MonitorTimerStop(hWnd);

	char	cRecvBuf[MONITOR_MAX_LOGSIZE];	// 受信バッファ(MAX 64kb)
	TCHAR	tcWriteLog[MONITOR_MAX_LOGSIZE];	// 表示データバッファ(MAX 64kb)
	DWORD   dwGetDataLen	= 0UL;			// 受信バッファから読み出せたサイズ
	DWORD	dwTextLen		= 0UL;			// ログの文字数
	DWORD	dwRecvLen		= g_MonitorComPort.GetReceiveDataLength();	//受信したデータサイズを格納

	memset( cRecvBuf, 0x00, MONITOR_MAX_LOGSIZE);
	memset( tcWriteLog, 0x00, MONITOR_MAX_LOGSIZE);

	// データの受信処理
	if( MONITOR_MAX_LOGSIZE < dwRecvLen ){
		dwRecvLen = MONITOR_MAX_LOGSIZE;
	}

	dwGetDataLen =  g_MonitorComPort.GetReceivedData(cRecvBuf, MONITOR_MAX_LOGSIZE, dwRecvLen);

	MultiByteToWideChar(CP_OEMCP,MB_PRECOMPOSED,cRecvBuf,strlen(cRecvBuf),tcWriteLog,sizeof( tcWriteLog));
	
	// 受信サイズ確認
	if( dwRecvLen > 0 ) {
		// 1byteでも受信して居た場合の処理
				
		// 文字の追加処理-------------------------------

		// 再描画停止
		SendMessage(g_hEditUpdateLog,WM_SETREDRAW,false,0);

		// 文字数の取得
		dwTextLen = SendMessage(g_hEditUpdateLog, WM_GETTEXTLENGTH, 0,0);

		// カーソルを末尾へ移動
		SendMessage(g_hEditUpdateLog,EM_SETSEL,dwTextLen,dwTextLen);

		// メッセージログへの追加
		SendMessage(g_hEditUpdateLog, EM_REPLACESEL, 0, (LPARAM)tcWriteLog);

		// 文字数上限値チェック-------------------------

		// 文字数の取得
		dwTextLen = SendMessage(g_hEditUpdateLog, WM_GETTEXTLENGTH, 0,0);

		// 上限値の確認
		long lOver = (long)dwTextLen - (long)MONITOR_MAX_LOGSIZE;
		if( lOver > 0 ) {
			// はみ出したログの削除処理

			// オーバーしている文字を選択状態にする
			SendMessage(g_hEditUpdateLog,EM_SETSEL, 0, lOver );

			// オーバーした文字を空文字に置き換える
			SendMessage(g_hEditUpdateLog, EM_REPLACESEL, 0, (LPARAM)_T(""));

			// カーソルを末尾へ移動
			SendMessage(g_hEditUpdateLog,EM_SETSEL,MONITOR_MAX_LOGSIZE,MONITOR_MAX_LOGSIZE);
		}

		// F/W更新ログのスクロール
		SendMessage(g_hEditUpdateLog, WM_VSCROLL, SB_BOTTOM, NULL);

		// 再描画再開
		SendMessage(g_hEditUpdateLog,WM_SETREDRAW,true,0);
		InvalidateRect(g_hEditUpdateLog,NULL,true);
	}


	// タイマ再開
	MonitorTimerStart(hWnd);
}

//-----------------------------------------------------------------------------
// @outline     シリアルモニタ用タイマイベント開始
// @explanation シリアルモニタ用のメッセージ表示用処理を開始します。
// @param       [IN] hWnd		イベントを発生させるウィンドウのハンドルを指定します。
// @return      void
//-----------------------------------------------------------------------------
UINT MonitorTimerStart(HWND hWnd){
	return SetTimer(hWnd, MONITOR_TIMER_ID, MONITOR_TIMER_INTERVAL, MonitorTimerProc);
}

//-----------------------------------------------------------------------------
// @outline     シリアルモニタ用タイマイベント停止
// @explanation シリアルモニタ用のメッセージ表示用処理を停止します。
// @param       [IN] hWnd		イベントを発生させるウィンドウのハンドルを指定します。
// @return      void
//-----------------------------------------------------------------------------
void MonitorTimerStop(HWND hWnd) {
	KillTimer(hWnd, MONITOR_TIMER_ID);
}

//-----------------------------------------------------------------------------
// @outline		UART通信設定のボー・レート設定および電圧設定データの設定を行います。
// @explanation	UART通信設定のボー・レート設定および電圧設定データの設定を行います。
// @param		[IN]bBaudRate   ボーレート値を指定します。(0x00 = 115,200bps | 0x01 = 250,000bps | 0x02 = 500,000 bps | 0x03 = 1,000,000 bps)
//              [IN]bVoltage    動作電圧を指定します。(例:)3.30V -> 33 -> 0x21:電圧値の小数第一位までの値を16進数として指定します。)
// @return		設定できた場合には SW_SUCCESS を、設定できない場合には SW_SUCCESS 以外を返します。
//-----------------------------------------------------------------------------
DWORD ComBaudRateSetProc(BYTE bBaudRate, BYTE bVoltage) 
{
    DWORD   dwRet       = SW_SUCCESS;   // 戻り値
    BOOL    fRetry      = FALSE;        // リトライフラグ
    DWORD   dwRetry     = 0UL;          // リトライした回数
    DWORD   dwCmdLength = 0UL;          // コマンド・フレームの長さ
    DWORD   dwLength    = 0UL;          // 受信バッファから読み出すサイズ

	DWORD	comRetryMax = 5UL;
	DWORD	transitionStartTime = 0UL;
	DWORD	transitionEndTime = 0UL;

	// コマンドクラス
	CCommandProcessor cmd;
	cmd.Initialize();

	// リセット
    if (FALSE == g_isInvertDTR) {
        g_MonitorComPort.SetDtr();
    }
    else {
		g_MonitorComPort.ClearDtr();
    }

	// TOOL0をLow
	g_MonitorComPort.SetBreak();

	// ウェイト
	Wait(1UL);

    // 受信バッファのクリア
    g_MonitorComPort.ClearReceiveBuffer();

    // リセット解除
    if (FALSE == g_isInvertDTR) {
        g_MonitorComPort.ClearDtr();
    }
    else {
        g_MonitorComPort.SetDtr();
    }

    // 100msec以内の処理時間測定開始
    transitionStartTime = timeGetTime();

	// リセット解除待ち
	Wait(1UL);

	// TOOL0をHigh
	g_MonitorComPort.ClearBreak();
		
    // ウェイト
    Wait(1UL);

	// モード設定1バイトデータ送信
	BYTE	bModeSetData = { COM_IF_TWO_WIRE }; // 2線UART
	dwRet = g_MonitorComPort.SendData(&bModeSetData, sizeof(bModeSetData));

    for (DWORD i = dwRetry = 0; i < comRetryMax; i++, dwRetry++) {

		// リトライフラグをクリア
        fRetry = FALSE;
	    
		// BaudRate設定コマンド送信
	    if (SW_SUCCESS == dwRet) {
		    // 送信前ウェイト:1msec
            Wait(1);

		    // BaudRate設定コマンドを作成する。
            dwCmdLength = 0UL;
            dwRet = cmd.MakeBaudRateCommand(bBaudRate, bVoltage, &dwCmdLength);
        
            // 送信
            if (SW_SUCCESS == dwRet) {
                dwRet = g_MonitorComPort.SendData(cmd.GetCommandFrames(), dwCmdLength);
            }

            // 100msec以内の処理時間測定終了
            transitionEndTime = timeGetTime();
	    }
        
        // BaudRate設定コマンドに対するステータス・フレームの受信待ち
        if (SW_SUCCESS == dwRet) {
            dwLength = 0UL;
            dwRet = WaitStatusFrames(&dwLength, 200UL);

            if (SW_ERROR_RECV_TIMEOUT == dwRet) {
                // タイムアウトならリトライフラグをセット
                fRetry = TRUE;
            }
        }

        if (FALSE == fRetry) {
            break;
        }
        else {
            // リトライウェイト
            Wait(100UL);
            // エラー値を戻す
            dwRet = SW_SUCCESS;
        }
    }

    // プログラミング・モードの制限時間チェック
    if (SW_SUCCESS == dwRet) {
        // ここまでの処理を100msec以内で完了していない場合はエラーとする。
        if (100UL < ( transitionEndTime - transitionStartTime)) {
            dwRet = SW_ERROR_MODE_CHANGE_TIMEOUT;
        }
    }

    return dwRet;
}

//-----------------------------------------------------------------------------
// @outline		指定された時間だけウェイトします。
// @explanation	指定された時間だけウェイトします。
// @param		[in]dwMilisecond    ウェイト時間を指定します。(msec)
// @return		実際にウェイトした時間を返します。
//-----------------------------------------------------------------------------
DWORD Wait(DWORD dwMilisecond)
{
	DWORD	dwStartTime;
	DWORD	dwEndTime;
	DWORD	dwPassTime;

	dwStartTime = timeGetTime();
	do {
		dwEndTime = timeGetTime();
	} while (dwPassTime = (dwEndTime - dwStartTime) <= dwMilisecond);

	return dwPassTime;
}

//-----------------------------------------------------------------------------
// @outline		データの受信待ちを行います。
// @explanation	データの受信待ちを行います。
// @param		[out]*dwLength  受信したデータの長さを格納します。
//              [in]dwTimeout   タイムアウト時間を指定します。(msec)
// @return		正常に受信できた場合には 0 を、受信できなかった場合には 0 以外を返します。
//-----------------------------------------------------------------------------
DWORD WaitStatusFrames(DWORD* dwLength, DWORD dwTimeout)
{
    DWORD   dwRet           = SW_SUCCESS;
    DWORD   dwReceiveLength = 0UL;
  	DWORD	dwStartTime     = 0UL;
	DWORD	dwPassTime      = 0UL;
    BYTE    bStx            = 0x00;
    DWORD   dwLen           = 0UL;

    *dwLength = 0UL;

    // 処理開始時間を取得
    dwStartTime = timeGetTime();
    do {
        // 受信データのサイズを取得する。
        dwReceiveLength = g_MonitorComPort.GetReceiveDataLength();

        // 2バイト以上受信していれば解析可能
        if (2 <= dwReceiveLength) {
            // STXとLENデータを取得
            dwRet = g_MonitorComPort.CheckDataFrame(&bStx, &dwLen);
            // チェック完了且つSTXデータの場合
            if (SW_SUCCESS == dwRet && STX == bStx) {
                // LENの内容から解析するフレーム全体のサイズを取得する。
                // LENの内容 + 4バイト分（STX、LEN、SUM、ETX or ETB）
                *dwLength = dwLen + 4;
            }
        }

        // 経過時間を取得する。
        dwPassTime = timeGetTime() - dwStartTime;

        // タイムアウト判定
        if (dwPassTime > dwTimeout) {
            dwRet = SW_ERROR_RECV_TIMEOUT;
            break;
        }
      } while (0UL == dwReceiveLength || 
               0UL == *dwLength || 
               *dwLength > dwReceiveLength);

    return dwRet;
}