/*
 * Copyright (c) 2012,2015 Renesas Electronics Corporation and TESSERA
 * TECHNOLOGY INC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//=============================================================================
// CComanndoProcessor NX
//=============================================================================
#include "stdwin.h"
#include "SerialPortProcessor.h"

#pragma comment(lib, ".\\res\\ftd2xx.lib")

//-----------------------------------------------------------------------------
// @outline		RXgN^
// @explanation	RXgN^
// @param		none
// @return		none
//-----------------------------------------------------------------------------
 CSerialPortProcessor::CSerialPortProcessor(void) 
    : m_hComPort(NULL), m_lpbReceivedData(NULL), m_dwReceivedDataLength(0), m_fReading(FALSE)
{ 
    // Mf[^̔rpMutex𐶐
    m_hMutex           = CreateMutex(NULL, TRUE, NULL); 

    // MXbhԃtOMutex𐶐
    m_hRecvThreadMutex = CreateMutex(NULL, TRUE, NULL);
}


//-----------------------------------------------------------------------------
// @outline		fXgN^
// @explanation	fXgN^
// @param		none
// @return		none
//-----------------------------------------------------------------------------
CSerialPortProcessor::~CSerialPortProcessor(void) 
{ 
    // |[gB
    Close();

    // Mf[^̔rpMutex̔j
	CloseHandle(m_hMutex);
	m_hMutex = NULL;

    // MXbhԃtOMutexj
	CloseHandle(m_hRecvThreadMutex);
	m_hRecvThreadMutex = NULL;
} 


//-----------------------------------------------------------------------------
// @outline		ʐMJn܂B
// @explanation	w肵|[gI[vĎMJn܂B
// @param		[IN]nPortNumber	|[gԍw肵܂B
//				[IN]portConfig	ʐMݒ\̂w肵܂B
// @return		Jnɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::Open(UINT nPortNumber, DCB* portConfig) 
{ 
	DWORD	dwRet = SW_SUCCESS;

    // Mf[^obt@m
    m_lpbReceivedData = (BYTE*)malloc(sizeof(BYTE) * RECEIVE_BUFF_SIZE);
    
    if (m_lpbReceivedData == NULL) {
        dwRet = SW_ERROR_INIT;
    }

    // w|[gJ 
    if (SW_SUCCESS == dwRet) {
        TCHAR portName[MAX_PATH];
        wsprintf(portName, _T("\\\\.\\COM%d"), nPortNumber);

        m_hComPort = CreateFile(portName,
							    GENERIC_READ|GENERIC_WRITE, 
							    0, 
							    NULL,
							    OPEN_EXISTING,
							    FILE_FLAG_OVERLAPPED, 
							    NULL);
	
	    if (INVALID_HANDLE_VALUE == m_hComPort) {
            dwRet = SW_ERROR_SERIAL_OPEN;
            m_hComPort = NULL;
	    }
    }

    // ʐMݒZbg
    if (SW_SUCCESS == dwRet) {
		if (SetCommState(m_hComPort, portConfig) == FALSE) {
            dwRet = ::GetLastError();
		}
	}

    if (SW_SUCCESS == dwRet) {
        // OVERLAPPED̏
        ZeroMemory(&m_ovlpWrite, sizeof(OVERLAPPED));

        // 񓯊ʐMp̏Cxg쐬(}jAZbg)
        m_ovlpWrite.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
        if (NULL == m_ovlpWrite.hEvent) {
            dwRet = GetLastError();
        }
    }
    
    // MXbh̐ƋN
    if (SW_SUCCESS == dwRet) {
		m_hThread = CreateThread(NULL,
								 0, 
								 GetThreadFunc,
								 (LPVOID)this,
								 CREATE_SUSPENDED,
								 &m_dwThreadId);
		if (m_hThread == NULL) {
            dwRet = ::GetLastError();
		}
        else {
            // XbhN
            ResumeThread(m_hThread);
        }
	}

    // G[
	if (SW_SUCCESS != dwRet && INVALID_HANDLE_VALUE != m_hComPort) {
		// Mf[^obt@mۂĂꍇ͉B
        if (NULL != m_lpbReceivedData) {
            free(m_lpbReceivedData);
            m_lpbReceivedData = NULL;
        }

        // G[|[gJĂꍇ͕B
		CloseHandle(m_hComPort);
		m_hComPort = NULL;
	}

    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		{[[gύX܂B
// @explanation	{[[gύX܂B
// @param		dwBaudRate      ύX{[[glw肵܂B
// @return		ύXɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::ChangeBaudRate(DWORD dwBaudRate)
{
    DWORD				dwRet   = SW_SUCCESS;       // ߂l
    DCB					buffDcb = {0};              // DCB\
	SERIAL_BAUD_RATE    sSB;						// {[[gi[
    DWORD				dummy;


    // nh̊mF
    if (NULL == m_hComPort) {
        dwRet = SW_ERROR_CHG_BAUDRATE;
    }
	
	//	{[[gݒ
	sSB.BaudRate = dwBaudRate;
    DeviceIoControl( m_hComPort, IOCTL_SERIAL_SET_BAUD_RATE,
                     &sSB, sizeof(sSB), 0, NULL,
                     &dummy, (LPOVERLAPPED) NULL);

	//	{[[gǂݎ
    DeviceIoControl( m_hComPort, IOCTL_SERIAL_GET_BAUD_RATE,
                     0, NULL, &sSB, sizeof(sSB),
                     &dummy, (LPOVERLAPPED) NULL);

	//	ݒ茋ʂ̊mF
	if( dwBaudRate != sSB.BaudRate )
		dwRet = SW_ERROR_CHG_BAUDRATE; 

    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		Mobt@NA܂B
// @explanation	Mobt@NA܂B
// @param		none
// @return		none
//-----------------------------------------------------------------------------
void CSerialPortProcessor::ClearReceiveBuffer()
{
    // ̓obt@̓eNA
    PurgeComm(m_hComPort, PURGE_RXCLEAR);
    
    // ǂݍ݃f[^NA
    clearReadData();
}


//-----------------------------------------------------------------------------
// @outline		ʐMI܂B
// @explanation	|[g܂B
// @param		none
// @return		none
//-----------------------------------------------------------------------------
void CSerialPortProcessor::Close() 
{ 
    DWORD       dwEvtMask;
	

    if (NULL != m_hComPort) {
	    if (m_hThread != NULL) {
            dwEvtMask   =   0;
            dwEvtMask|=EV_RXCHAR;

            // ǂ݂ݒtONAB
            updateReadingStatus(FALSE);

            // }XN邱ƂWaitCommEvent()֐畜AB
            SetCommMask (m_hComPort, dwEvtMask);

            // XbhIҋ@B
            do {
                GetExitCodeThread(m_hThread, &m_dwThreadId);
            } while (m_dwThreadId == STILL_ACTIVE);

            // obt@NAB
	        PurgeComm( m_hComPort,
                      PURGE_TXABORT | PURGE_RXABORT |
                      PURGE_TXCLEAR | PURGE_RXCLEAR);
	        // XbhnhB
            CloseHandle (m_hThread);
            m_hThread = NULL;
        }

        // 񓯊ʐMp̏Cxg̔j
        if (NULL != m_ovlpWrite.hEvent) {
            ResetEvent(m_ovlpWrite.hEvent);
            CloseHandle(m_ovlpWrite.hEvent);
            m_ovlpWrite.hEvent = NULL;
        }

        // |[gB
        CloseHandle(m_hComPort); 
		m_hComPort = NULL;
    }

    // Mf[^obt@mۂĂꍇ͉B
    if (NULL != m_lpbReceivedData) {
        free(m_lpbReceivedData);
        m_lpbReceivedData = NULL;
    }
} 


//-----------------------------------------------------------------------------
// @outline		MXbh֐擾܂B
// @explanation	MXbh֐擾܂B
// @param		[IN]lpParameter	Jñp[^w肵܂B
// @return		MXbhƂē삷֐̃AhXԂ܂B
//-----------------------------------------------------------------------------
DWORD WINAPI CSerialPortProcessor::GetThreadFunc(LPVOID lpvParameter) 
{ 
    return ((CSerialPortProcessor*)lpvParameter)->receiveData(); 
}


//-----------------------------------------------------------------------------
// @outline		MXbh̓Ԃ擾܂B
// @explanation	MXbh̓Ԃ擾܂B
// @param		none
// @return		삵Ăꍇɂ TRUE A삵ĂȂꍇɂ FALSE Ԃ܂B
//-----------------------------------------------------------------------------
BOOL CSerialPortProcessor::IsReading()
{
    CScopedLock lock(m_hRecvThreadMutex);

    return m_fReading;
}


//-----------------------------------------------------------------------------
// @outline		MXbh̓ԂXV܂B
// @explanation	MXbh̓ԂXV܂B
// @param		[IN]fReading    삵Ăꍇɂ TRUEA삵ĂȂꍇɂ FALSE w肵܂B
// @return      none
//-----------------------------------------------------------------------------
void CSerialPortProcessor::updateReadingStatus(BOOL fReading)
{
    CScopedLock lock(m_hRecvThreadMutex);

    m_fReading = fReading;
}


//-----------------------------------------------------------------------------
// @outline		Ms܂B
// @explanation	Ms܂B
// @param		none
// @return		XbhI0Ԃ܂B
//-----------------------------------------------------------------------------
DWORD WINAPI CSerialPortProcessor::receiveData()
{
    OVERLAPPED  ovlpdRead = {0};                // 񓯊ǂݍݗp\
    DWORD       dwErrorStatus;                  // G[
    DWORD       dwEvent;                        // ʐMfoCX̃Cxg
    DWORD       dwRead;                         // ǂݍ񂾃f[^
    DWORD       dwEndtime;                      // ʐMԂ𔻒肷鎞(msec)
    DWORD       dwEvtMask;                      // MCxg
    COMSTAT     dwComStatus;                    // |[g̏ԂɊւ
    BYTE        buffData[INTERNAL_BUFF_SIZE];   // Xbh̃f[^obt@TCY

    // MCxg}XN
    dwEvtMask   =   0;
    dwEvtMask   |=  EV_RXCHAR;


    // VA|[g̎MCxgĎݒ
    SetCommMask (m_hComPort, dwEvtMask);

    // 񓯊ʐMp̏Cxg쐬(}jAZbg)
    ovlpdRead.hEvent = CreateEvent(NULL, true, false,NULL);

    updateReadingStatus(TRUE);

    while (TRUE == IsReading()) {
        // MCxgҋ@
        //DTRACE(_T("thread wait\r\n"));
        if (WaitCommEvent(m_hComPort, &dwEvent, NULL)) {
            // MTCY擾邽߂ɃG[Xe[^X擾B
            ClearCommError(m_hComPort, &dwErrorStatus, &dwComStatus);

            //DTRACE(_T("cbInQue=%u\r\n"), dwComStatus.cbInQue);

            // M
            if ((dwEvent & EV_RXCHAR) && dwComStatus.cbInQue) {
                // obt@NA
                ZeroMemory(buffData, sizeof(buffData));
                // 񓯊ǂݏo(ReadFile()֐̓ubNȂ)
                if (FALSE == ReadFile(m_hComPort, buffData, dwComStatus.cbInQue, &dwRead, &ovlpdRead)) {
                    // ǂݏoȂꍇ
                    dwRead = 0;
                    if (GetLastError() == ERROR_IO_PENDING) {
                        // M̖ʐM^CAEgԂݒ(msec)
                        dwEndtime = GetTickCount() + 1000;
                        // 񓯊ǂݏo
                        // GetOverlappedResult()֐̓ubNȂB
                        // G[͏wŎM^CAEgƂ邽߉ȂB
                        while (FALSE == GetOverlappedResult(m_hComPort, &ovlpdRead, &dwRead, FALSE)) {
                            // G[l擾
                            DWORD dwErr = GetLastError();
                            if (ERROR_IO_INCOMPLETE != dwErr) {
                                // G[
                                //DTRACE(_T("error\r\n"));
                                break;
                            }
                            if (GetTickCount() > dwEndtime) {
                                // ʐMԂ̃^CAEg
                                //DTRACE(_T("no communication\r\n"));
                                break;
                            }
                        }
                    }
                    else {
                        dwRead = 0;
                    }
                }

                if(dwRead > 0UL) {
                    // obt@O
                    //DTRACE(_T("baffaring!\r\n"));
                    setReadData(buffData, dwRead);
                }
            }
        }
        // 񓯊ʐMp̏Cxg̃Zbg
        ResetEvent(ovlpdRead.hEvent);
    }
    
    // 񓯊ʐMp̏Cxgj
    if (NULL != ovlpdRead.hEvent) {
        CloseHandle(ovlpdRead.hEvent);
    }

    // Mobt@̃NA
    PurgeComm(m_hComPort, PURGE_RXCLEAR);

    //DTRACE(_T("ReceiveThread exit!"));
  
    return 0;
}


//-----------------------------------------------------------------------------
// @outline     Mf[^̃obt@Os܂B
// @explanation	Mf[^̃obt@Os܂B
// @param		[IN]lpvBuffer   obt@̃|C^w肵܂B
//              [IN]nReadSize   obt@̃TCYw肵܂B
// @return		obt@Oɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::setReadData(LPVOID lpvBuffer, ULONG nReadSize)
{
    DWORD   dwRet   =   SW_SUCCESS;

    // r
    CScopedLock lock(m_hMutex);
    
    if (RECEIVE_BUFF_SIZE > (m_dwReceivedDataLength + nReadSize)) {

        // Mf[^֒ǉB
        memcpy_s((m_lpbReceivedData + m_dwReceivedDataLength),
                 RECEIVE_BUFF_SIZE, 
                 lpvBuffer, 
                 nReadSize);
    
        // Mf[^i[TCYXVB
        m_dwReceivedDataLength += nReadSize;
    }
    else {
        dwRet = SW_ERROR_BUFFER_OVER_FLOW;
    }
    
    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline     Mf[^NA܂B
// @explanation	Mf[^NA܂B
// @param		none
// @return		none
//-----------------------------------------------------------------------------
void CSerialPortProcessor::clearReadData()
{
    // r
    CScopedLock lock(m_hMutex);

    // Mf[^obt@NA
    // Mf[^obt@Ɋi[Ăf[^TCY0ɂ
    m_dwReceivedDataLength = 0UL;
}


//-----------------------------------------------------------------------------
// @outline     Mf[^̃TCY擾܂B
// @explanation	Mf[^̃TCY擾܂B
// @param		none
// @return		Mf[^̃TCYԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::GetReceiveDataLength()
{
    // r
    CScopedLock lock(m_hMutex);

    DWORD   dwLength = 0UL;

    dwLength = m_dwReceivedDataLength;

    return dwLength;
}


//-----------------------------------------------------------------------------
// @outline     f[^Et[STXLEN擾܂B
// @explanation	f[^Et[STXLEN擾܂B
// @param		[OUT] bStx  STXli[܂B
//              [OUT] dwLen LENli[܂B
// @return		Mf[^̃TCYԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::CheckDataFrame(BYTE* bStx, DWORD* dwLen)
{
    // r
    CScopedLock lock(m_hMutex);
    
    DWORD   dwRet = SW_SUCCESS;

    *bStx  = m_lpbReceivedData[0];
    *dwLen = m_lpbReceivedData[1];

    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		Mς݃f[^擾܂B
// @explanation	Mς݃f[^擾܂B
// @param		[OUT]lpvBuffer      Mf[^i[|C^w肵܂B
//              [IN]dwBufferSize    Mf[^i[obt@̍őTCYw肵܂B
//              [IN]dwReadLength    ǂݏoTCYw肵܂B
// @return		ǂݏoTCYԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::GetReceivedData(LPVOID lpvBuffer, DWORD dwBufferSize, DWORD dwReadLength)
{
    DWORD	dwLength        = 0UL; 
    DWORD   dwRemainSize    = 0UL;

    // r
    CScopedLock lock(m_hMutex);
    
    if (dwReadLength <= m_dwReceivedDataLength) {
        // Mf[^Ɋi[Ăf[^Rs[B
        memcpy_s(lpvBuffer, 
                 dwBufferSize, 
                 m_lpbReceivedData, 
                 dwReadLength);


        dwRemainSize = m_dwReceivedDataLength - dwReadLength;

        memmove_s(m_lpbReceivedData,
                  INTERNAL_BUFF_SIZE,
                  m_lpbReceivedData + dwReadLength,
                  dwRemainSize);


        m_dwReceivedDataLength -= dwReadLength;
        dwLength = dwReadLength;
    }
    else {
        dwLength  = 0;
    }
      
    return dwLength; 
}


//-----------------------------------------------------------------------------
// @outline		f[^𑗐M܂B
// @explanation	f[^𑗐M܂B
// @param		[IN]lpvBuffer		Mobt@̃|C^w肵܂B
//				[IN]nToWriteBytes	Mf[^̃TCYw肵܂B
// @return		f[^̑Mɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::SendData(LPVOID lpvBuffer, DWORD dwToWriteBytes) 
{   
    DWORD       dwWritten;                              // ݃oCg
    DWORD       dwWriteTimeOut  = SEND_DATA_TIMEOUT;    // ݎ̃^CAEg
    DWORD       dwRet           = SW_SUCCESS;           // ߂l
    
    // 񓯊ʐMp̏CxgZbg
    SetEvent(m_ovlpWrite.hEvent);

    // Cxg̃ZbgɎs
    if (FALSE == SetEvent(m_ovlpWrite.hEvent)) {
        dwRet = SW_ERROR_INIT;
    }
    else {
        // 񓯊ŏ݁iǂɊւ炸ɐ䂪߂j
        if (FALSE == WriteFile(m_hComPort, lpvBuffer, dwToWriteBytes, &dwWritten, &m_ovlpWrite)) {
            // 񓯊ł̏݊`FbN
            // ҂Ȃ ERROR_IO_PENDING == GetLastError()ƂȂĂA
            // ȊO͏݃G[
            if (ERROR_IO_PENDING != GetLastError()) {
                // ݊҂ȊÕG[
                dwRet = SW_ERROR_SEND_DATA;
            }
            else {
                // ݊҂Cxg̔K̃^CAEgԓőҋ@
                dwRet = WaitForSingleObject(m_ovlpWrite.hEvent, dwWriteTimeOut);
                switch( dwRet ) {
                case WAIT_OBJECT_0:
                    // 񓯊݂̊`FbN
                    if (FALSE == GetOverlappedResult(m_hComPort, &m_ovlpWrite, &dwWritten, FALSE)) {
                        dwRet = SW_ERROR_SEND_DATA;
                    }
                    else {
                        // ݃f[^̃`FbN
                        if (dwToWriteBytes == dwWritten) {
                            dwRet = SW_SUCCESS;
                        } 
                        else {
                            dwRet = SW_ERROR_SEND_DATA;
                        }                      
                    }
                    break;

                default:
                    // ݃^CAEg
                    dwRet = SW_ERROR_SEND_TIMEOUT;
                }
            }
        }
        else {
            // ݂I
            dwRet = SW_SUCCESS;
        }
        // CxgZbg
        ResetEvent(m_ovlpWrite.hEvent);
    }

    return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		M𒆒fMؒfԂɂ܂B
// @explanation	M𒆒fMؒfԂɂ܂B
// @param		none
// @return		ؒfԂɂłꍇɂ SW_SUCCESS AłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::SetBreak()
{
	DWORD	dwRet = SW_SUCCESS;

	if (EscapeCommFunction(m_hComPort, SETBREAK) == FALSE) {
		dwRet = GetLastError();
	}

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		M̐ؒfԂĕMĊJ܂B
// @explanation	M̐ؒfԂĕMĊJ܂B
// @param		none
// @return		ĊJłꍇɂ SW_SUCCESS AłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::ClearBreak()
{
	DWORD	dwRet = SW_SUCCESS;

	if (EscapeCommFunction(m_hComPort, CLRBREAK) == FALSE) {
		dwRet = GetLastError();
	}

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		DTRi f[^[jM𑗐M܂B
// @explanation	DTRi f[^[jM𑗐M܂B
// @param		none
// @return		DTRM𑗐Młꍇɂ SW_SUCCESS AłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::SetDtr()
{
	DWORD	dwRet = SW_SUCCESS;

	if (EscapeCommFunction(m_hComPort, SETDTR) == FALSE) {
		dwRet = GetLastError();
	}

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		DTRi f[^[jM܂B
// @explanation	DTRi f[^[jM܂B
// @param		none
// @return		DTRMłꍇɂ SW_SUCCESS AłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::ClearDtr()
{
	DWORD	dwRet = SW_SUCCESS;

	if (EscapeCommFunction(m_hComPort, CLRDTR) == FALSE) {
		dwRet = GetLastError();
	}

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		VA|[g擾܂B
// @explanation	VA|[g擾܂B
// @param		serialPortInfoList
// @return		擾ɐꍇɂ SW_SUCCESS Asꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::GetSerialPortInfo(vector<SERIALPORTINFO> *serialPortInfoList)
{
	GUID				classGuid[1];                           // PortsNXGUIDl
	DWORD				dwRequiredSize;                         // PortsNXɊ֘AĂGUID
	BOOL				fRet;                                   // ֐̖߂l
	HDEVINFO			deviceInfoSet   = INVALID_HANDLE_VALUE; // HDEVINFO\
	SP_DEVINFO_DATA		deviceInfoData;                         // SP_DEVINFO_DATA\
	DWORD				dwMemberIndex   = 0;                    // PorsNXɑfoCX̃JE^
    DWORD               dwRet           = SW_SUCCESS;           // ߂l

	// 
	deviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);

	// PortsNXGUID擾
	fRet = SetupDiClassGuidsFromName(_T("PORTS"),
									 (LPGUID)&classGuid,
									 1,
									 &dwRequiredSize);
	if (FALSE == fRet) {
        // VA|[g̎擾s
        //DTRACE(_T("%s:ecode = %d\r\n"), CA2T(__FUNCTION__),::GetLastError());
        dwRet = SW_ERROR_SERIAL_INFO;
	}
    else {
	    // PortsNX̃foCXZbg̃nh擾
	    deviceInfoSet = SetupDiGetClassDevs(&classGuid[0], 
										    NULL, 
										    NULL, 
										    DIGCF_PRESENT | DIGCF_PROFILE); // ݃VXeɐڑĂfoCX̂ | ݂̃n[hEFA vt@CɊ܂܂foCX̂

	    if (deviceInfoSet != INVALID_HANDLE_VALUE) {
		    dwMemberIndex = 0;
		    // foCXGg
		    while (SetupDiEnumDeviceInfo(deviceInfoSet, 
									     dwMemberIndex++,
									     &deviceInfoData)) {

			    TCHAR szFriendlyName[MAX_PATH];
			    TCHAR szPortName[MAX_PATH];
			    DWORD dwReqSize = 0;
			    DWORD dwPropType;
			    DWORD dwType = REG_SZ;
			    HKEY  hKey = NULL;

			    // foCX̏ڍ׏擾
			    fRet = SetupDiGetDeviceRegistryProperty(deviceInfoSet,
			                                            &deviceInfoData,
			                                            SPDRP_FRIENDLYNAME,
			                                            &dwPropType,
			                                            (LPBYTE)szFriendlyName,
			                                            sizeof(szFriendlyName),
			                                            &dwReqSize);
                if (FALSE == fRet) {
                    // ڍ׏̎擾ɎsA܂݂͑Ȃꍇ͖
                    continue;
                }

			    // WXgڍ׏擾
			    hKey = SetupDiOpenDevRegKey(deviceInfoSet,
			                                &deviceInfoData,
			                                DICS_FLAG_GLOBAL,
			                                0, 
										    DIREG_DEV, 
										    KEY_READ);
			
			    if (INVALID_HANDLE_VALUE != hKey) {
				    long lRet;
				    dwReqSize = sizeof(szPortName);
				    // WXgL[|[g擾
                    // |[g擾Ă邾Ŗ߂l͍lȂB
				    lRet = RegQueryValueEx(hKey,
				                           _T("PortName"),
				                           0,
				                           &dwType,
				                           (LPBYTE) &szPortName,
				                           &dwReqSize);
				
                    // WXg
				    RegCloseKey(hKey);
			    }

                // COM|[g݂ΏXg֊i[B
			    if (_tcsncicmp(szPortName, _T("COM"), 3) == 0) {
				    int port = _tstoi(&szPortName[3]);
				    // VA|[g\̂𐶐ăXgɒǉ
				    SERIALPORTINFO serialPortInfo;
				    serialPortInfo.portNumber = _tstoi(&szPortName[3]);
				    _tcscpy_s(serialPortInfo.description, MAX_PATH, szFriendlyName);
				    _tcscpy_s(serialPortInfo.portName, MAX_PATH, szPortName);
                    serialPortInfoList->push_back(serialPortInfo);
			    }
		    }
	    }
    }

    // foCXZbgjB
	SetupDiDestroyDeviceInfoList(deviceInfoSet);

	return dwRet;
}


//-----------------------------------------------------------------------------
// @outline		DTRM̐ݒl𔽓]܂B
// @explanation	݂DTRM̐ݒl𖳏ɔ]܂B
// @param		[pDevChangeInfo] foCXύXw肵܂B
// @return		Iꍇɂ SW_SUCCESSAIłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::InvertDTR(DEVCHANGEINFO *pDevChangeInfo)
{
    DWORD               dwRet               = SW_SUCCESS;   // ߂l
	FT_HANDLE           fthandle            = NULL;         // FTDIfoCX̃nh
	FT_STATUS           status              = FT_OK;        // FTDIAPI̖߂l
    DWORD               dwDevCount;                         // FTDIfoCX̐
    LONG                lComPort;                           // COM|[gԍ
    HANDLE              hDevChangeEvent     = NULL;         // foCXύXCxg

    // FTDĨfoCX擾
    if (SW_SUCCESS == dwRet) {
	    status = FT_ListDevices(&dwDevCount, NULL, FT_LIST_NUMBER_ONLY);
        DTRACE(_T("## dwDevCount = %d\r\n"), dwDevCount);
        if (FT_OK != status) {
            DTRACE(_T("%d = FT_ListDevices()\r\n"), status);
            dwRet = SW_ERROR_CHG_EEPROM;
        }
    }

    if (SW_SUCCESS == dwRet) {
        for (DWORD i = 0; i < dwDevCount; i++) {
            // hCoI[vB
	        status = FT_Open(int(i), &fthandle);
	        if(status != FT_OK) {
		        DTRACE(_T("%d = FT_Open(%d)\r\n"), status, i);
                dwRet = SW_ERROR_CHG_EEPROM;
                break;
	        }
            else {
                // COM|[gԍ擾
                status = FT_GetComPortNumber(fthandle, &lComPort);
                if (FT_OK != status) {
                    DTRACE(_T("%d = FT_GetComPortNumber()\r\n"), status);
                    dwRet = SW_ERROR_CHG_EEPROM;
                    status = FT_Close(fthandle);
                    fthandle = NULL;
                    break;
                }
                else {
                    if (pDevChangeInfo->PortNumber == lComPort) {
                        // ΏۃfoCX
                        break;
                    }
                    else {
                        // ΏۃfoCXłȂ̂ŃN[Y
                        status = FT_Close(fthandle);
                        fthandle = NULL;
                        if (FT_OK != status) {
                            dwRet = SW_ERROR_CHG_EEPROM;
                        }
                    }
                }
            }
        }
    }

    // EEPROMւ̃ANZX
    if (SW_SUCCESS == dwRet) {
        switch (getFTDeviceType(pDevChangeInfo->PortNumber)) {
            case FT_DEVICE_232R:
                DTRACE(_T("FT_DEVICE_232R\r\n"));
                dwRet = invertDTR_Ft232r(fthandle);
                break;
            case FT_DEVICE_X_SERIES:
                DTRACE(_T("FT_DEVICE_X_SERIES\r\n"));
                dwRet = invertDTR_FtX(fthandle);
                break;
            default:
                DTRACE(_T("Not Supported Device\r\n"));
                dwRet = SW_ERROR_CHG_EEPROM;
                break;
        }
    }

    if (SW_SUCCESS == dwRet) {
        // foCXύXCxg𐶐
        hDevChangeEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
        if (NULL != hDevChangeEvent) {
            // foCXύXCxgƃtOZbg
            pDevChangeInfo->hNotify = hDevChangeEvent;
            pDevChangeInfo->IsDeviceChange = TRUE;

            // }
            status = FT_CyclePort(fthandle);
            if (FT_OK == status) {
                // foCXύX҂i^CAEgtj
                dwRet = WaitForSingleObject(pDevChangeInfo->hNotify, DEVICE_CHG_TIMEOUT);
                DTRACE(_T("%d = WaitForSingleObject()\r\n"), dwRet);
            }
            else {
                DTRACE(_T("%d = FT_CyclePort()\r\n"), status);
                dwRet = SW_ERROR_CHG_EEPROM;
            }
        }
        else {
            DTRACE(_T("CreateEvent error.\r\n"));
            dwRet = SW_ERROR_CHG_EEPROM;
        }
    }

    if (NULL != fthandle) {
        // FTDĨhCoN[YB
        status = FT_Close(fthandle);
        if (FT_OK != status) {
            DTRACE(_T("%d = FT_Close()\r\n"), status);
            dwRet = SW_ERROR_CHG_EEPROM;
        }
        fthandle = NULL;
    }

    // foCXύXCxg̔j
    if (NULL != hDevChangeEvent) {
        CloseHandle(hDevChangeEvent);
        hDevChangeEvent = NULL;
    }

    return dwRet;
}

//-----------------------------------------------------------------------------
// @outline		foCX^Cv擾܂B
// @explanation	݂̐ڑĂfoCX̃foCX^CvԂ܂B
// @param		[ComPort] ڑĂComPortw肵܂B
// @return		foCX^CvԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::getFTDeviceType(UINT ComPort)
{
    FT_STATUS   ftStatus;                   // FTDIAPI̖߂l
    FT_HANDLE   ftHandle;                   // FTDIfoCX̃nh
    DWORD       numDevs;                    // FTDIfoCX̐
    DWORD       Flags;                      // FTDIfoCXtO
    DWORD       ID;                         // FTDIfoCXID
    DWORD       Type = FT_DEVICE_UNKNOWN;   // FTDIfoCX^Cv
    DWORD       LocId;                      // FTDIfoCXP[VID
    char        SerialNumber[16];           // VAio[
    char        Description[64];            // 
    LONG        lComPort;                   // COM|[gԍ

    // foCX̃Xg쐬
    ftStatus = FT_CreateDeviceInfoList(&numDevs);
    if (ftStatus == FT_OK) {
        DTRACE(_T("Number of devices is %d\n"), numDevs);
    }

    if (numDevs > 0) {
        for (DWORD i = 0; i < numDevs; i++) {
            // foCX̏擾
            ftStatus = FT_GetDeviceInfoDetail(i, &Flags, &Type, &ID, &LocId, SerialNumber, Description, &ftHandle);
            if (ftStatus == FT_OK) {
                // COM|[gԍ擾
                ftStatus = FT_GetComPortNumber(ftHandle, &lComPort);
                if (ftStatus == FT_OK && lComPort == ComPort) {
                    // ΏۃfoCX
                    DTRACE(_T("Dev 0:\n"));
                    DTRACE(_T(" Flags=0x%x\n"), Flags);
                    DTRACE(_T(" Type=0x%x\n"), Type);
                    DTRACE(_T(" ID=0x%x\n"), ID);
                    DTRACE(_T(" LocId=0x%x\n"), LocId);
                    DTRACE(_T(" SerialNumber=%s\n"), SerialNumber);
                    DTRACE(_T(" Description=%s\n"), Description);
                    DTRACE(_T(" ftHandle=0x%x\n"), ftHandle);
                    break;
                } else {
                    // ΏۃfoCXł͂Ȃ
                    Type = FT_DEVICE_UNKNOWN;
                }
            } else {
                // foCX̏擾łȂ
                Type = FT_DEVICE_UNKNOWN;
            }
        }
    }

    return Type;
}

//-----------------------------------------------------------------------------
// @outline		FT232RDTRM̐ݒl𔽓]܂B
// @explanation	݂DTRM̐ݒl𖳏ɔ]܂B
// @param		[fthandle] FT_HANDLEw肵܂B
// @return		Iꍇɂ SW_SUCCESSAIłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::invertDTR_Ft232r(FT_HANDLE fthandle)
{
    DWORD               dwRet               = SW_SUCCESS;   // ߂l
	FT_STATUS           status              = FT_OK;        // FTDIAPI̖߂l
	char                manufacturer[64];                   // }jt@N`
	char                manufacturerId[64];                 // }jt@N`ID
	char                description[64];                    // 
	char                serialNumber[64];                   // VAio[
	FT_EEPROM_HEADER    ft_eeprom_header;                   // FTDIEEPROMwb_\
    FT_EEPROM_232R      ft_eeprom232r;                      // FT232REEPROM\


    // wb_Zbg
	ft_eeprom_header.deviceType = FT_DEVICE_232R;
	ft_eeprom232r.common = ft_eeprom_header;
	ft_eeprom232r.common.deviceType = FT_DEVICE_232R;

    // EEPROM̓ǂݏo
    if (SW_SUCCESS == dwRet) {
	    status = FT_EEPROM_Read(    fthandle,
                                    &ft_eeprom232r,
                                    sizeof(ft_eeprom232r),
                                    manufacturer,
                                    manufacturerId,
                                    description,
                                    serialNumber);
        if (FT_OK != status) {
            DTRACE(_T("%d = FT_EEPROM_Read()\r\n"), status);
            dwRet = SW_ERROR_CHG_EEPROM;
        }
    }

    if (SW_SUCCESS == dwRet) {
        // ݒl𔽓]B
	    ft_eeprom232r.InvertDTR = !ft_eeprom232r.InvertDTR;
        // EEPROM֏
	    status = FT_EEPROM_Program( fthandle,
                                    &ft_eeprom232r,
                                    sizeof(ft_eeprom232r),
                                    manufacturer, 
                                    manufacturerId, 
                                    description, 
                                    serialNumber);
        if (FT_OK != status) {
            DTRACE(_T("%d = FT_EEPROM_Program()\r\n"), status);
            dwRet = SW_ERROR_CHG_EEPROM;
        }
	}

	return dwRet;
}

//-----------------------------------------------------------------------------
// @outline		FT_X_SERIESDTRM̐ݒl𔽓]܂B
// @explanation	݂DTRM̐ݒl𖳏ɔ]܂B
// @param		[fthandle] FT_HANDLEw肵܂B
// @return		Iꍇɂ SW_SUCCESSAIłȂꍇɂ SW_SUCCESS ȊOԂ܂B
//-----------------------------------------------------------------------------
DWORD CSerialPortProcessor::invertDTR_FtX(FT_HANDLE fthandle)
{
    DWORD               dwRet               = SW_SUCCESS;   // ߂l
	FT_STATUS           status              = FT_OK;        // FTDIAPI̖߂l
	char                manufacturer[64];                   // }jt@N`
	char                manufacturerId[64];                 // }jt@N`ID
	char                description[64];                    // 
	char                serialNumber[64];                   // VAio[
	FT_EEPROM_HEADER    ft_eeprom_header;                   // FTDIEEPROMwb_\
    FT_EEPROM_X_SERIES  ft_eepromX;                         // FT_X_SERIESEEPROM\


    // wb_Zbg
	ft_eeprom_header.deviceType = FT_DEVICE_X_SERIES;
	ft_eepromX.common = ft_eeprom_header;
	ft_eepromX.common.deviceType = FT_DEVICE_X_SERIES;

    // EEPROM̓ǂݏo
    if (SW_SUCCESS == dwRet) {
	    status = FT_EEPROM_Read(    fthandle,
                                    &ft_eepromX,
                                    sizeof(ft_eepromX),
                                    manufacturer,
                                    manufacturerId,
                                    description,
                                    serialNumber);
        if (FT_OK != status) {
            DTRACE(_T("%d = FT_EEPROM_Read()\r\n"), status);
            dwRet = SW_ERROR_CHG_EEPROM;
        }
    }

    if (SW_SUCCESS == dwRet) {
        // ݒl𔽓]B
	    ft_eepromX.InvertDTR = !ft_eepromX.InvertDTR;
        // EEPROM֏
	    status = FT_EEPROM_Program( fthandle,
                                    &ft_eepromX,
                                    sizeof(ft_eepromX),
                                    manufacturer, 
                                    manufacturerId, 
                                    description, 
                                    serialNumber);
        if (FT_OK != status) {
            DTRACE(_T("%d = FT_EEPROM_Program()\r\n"), status);
            dwRet = SW_ERROR_CHG_EEPROM;
        }
	}

	return dwRet;
}
