/*
 * Copyright (c) 2012,2015 Renesas Electronics Corporation and TESSERA
 * TECHNOLOGY INC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//=============================================================================
// FileName : SerialPortProcessor.h
//=============================================================================
#pragma once

//=============================================================================
// CN[h
//=============================================================================
#include "Common.h"
#include <SetupAPI.h>
#include "ScopedLock.h"
#include <winioctl.h>
#include "ftd2xx.h"

//=============================================================================
// N
//=============================================================================
#pragma comment(lib, "setupapi.lib")

//=============================================================================
// 萔`
//=============================================================================
#define     SEND_DATA_TIMEOUT			(1000 * 1)      // f[^M̃^CAEg(msec)
#define     RECEIVE_BUFF_SIZE			(1024)          // Mf[^̃obt@TCY
#define     INTERNAL_BUFF_SIZE			(512)           // Mf[^̃obt@TCY
#define		IOCTL_SERIAL_SET_BAUD_RATE	CTL_CODE(FILE_DEVICE_SERIAL_PORT, 1, \
											METHOD_BUFFERED,FILE_ANY_ACCESS)	// {[[gݒs邽߂̐R[h
#define		IOCTL_SERIAL_GET_BAUD_RATE	CTL_CODE(FILE_DEVICE_SERIAL_PORT,20, \
											METHOD_BUFFERED,FILE_ANY_ACCESS)	// {[[g擾s邽߂̐R[h
#define     DEVICE_CHG_TIMEOUT          (1000 * 15)     // foCXύXʒm̃^CAEg(msec)

//=============================================================================
// NX`
//=============================================================================
//-----------------------------------------------------------------------------
// @outline		CSerialPortProcessor NX
// @explanation	VAE|[g̐sNXłB
//-----------------------------------------------------------------------------
class CSerialPortProcessor 
{ 
public: 
	// RXgN^
    CSerialPortProcessor(void); 
	// fXgN^
    ~CSerialPortProcessor(void); 

    // ʐMJn܂B
	DWORD			Open(UINT nPortNumber, DCB* portConfig); 
    
    // COM |[gI 
    void			Close(); 
    
    // Mς݃f[^擾܂B
    DWORD           GetReceivedData(LPVOID lpvBuffer, DWORD dwBufferSize, DWORD dwReadLength);

    // f[^𑗐M܂B
    DWORD			SendData(LPVOID lpvBuffer, DWORD dwToWriteBytes); 
	
    // M𒆒fMؒfԂɂ܂B
	DWORD			SetBreak();
	
    // M̐ؒfԂĕMĊJ܂B
	DWORD			ClearBreak();
	
    // DTRi f[^[jM𑗐M܂B
	DWORD			SetDtr();
	
    // DTRi f[^[jM܂B
	DWORD			ClearDtr();
	
    // VA|[g擾܂B
    static DWORD	GetSerialPortInfo(vector<SERIALPORTINFO> *serialPortInfoList);
    
    // Mf[^̃TCY擾܂B
    DWORD           GetReceiveDataLength();

    // MXbh̓Ԃ擾܂B
    BOOL            IsReading();

    // f[^Et[STXLEN擾܂B
    DWORD           CheckDataFrame(BYTE* bStx, DWORD* dwLen);

    // {[[gύX܂B
    DWORD           ChangeBaudRate(DWORD dwBaudRate);

    // Mobt@NA܂B
    void            ClearReceiveBuffer();

    // DTRM̐ݒl𔽓]܂B
    static DWORD InvertDTR(DEVCHANGEINFO *pDevChangeInfo);
  
private:
    // MXbh̓ǂݍݒtO
    BOOL            m_fReading;
	
    // VA|[g̃nh 
    HANDLE			m_hComPort;
	
    // MXbh̃nh 
    HANDLE			m_hThread;
	
    // MXbh̃XbhID 
    DWORD			m_dwThreadId;
	
    // Mf[^̔rpMutex
    HANDLE			m_hMutex;
    
    // MXbhԃtOMutex
    HANDLE          m_hRecvThreadMutex;
    
    // MXbhԃtOXV܂B
    void            updateReadingStatus(BOOL fReading);

	// Mf[^obt@
    BYTE*			m_lpbReceivedData;
	
    // Mf[^obt@Ɋi[Ăf[^TCY
    DWORD			m_dwReceivedDataLength;
    
    // 񓯊ʐMpOVERLAPPED\(ݗp)
    OVERLAPPED      m_ovlpWrite;

    // MXbh֐擾܂B
    static DWORD	WINAPI GetThreadFunc(LPVOID lpvParameter); 
	
    // Ms܂B
    DWORD			WINAPI receiveData();
    
    // Mf[^̃obt@Os܂B
    DWORD           setReadData(LPVOID lpvBuffer, ULONG nReadSize);

    // Mf[^NA܂B
    void            clearReadData();

    // foCX^Cv擾܂B
    static DWORD    getFTDeviceType(UINT ComPort);

    // FT232RDTRM̐ݒl𔽓]܂B
    static DWORD    invertDTR_Ft232r(FT_HANDLE fthandle);

    // FT_X_SERIESDTRM̐ݒl𔽓]܂B
    static DWORD    invertDTR_FtX(FT_HANDLE fthandle);

};
