/*
 * Copyright (c) 2012,2015 Renesas Electronics Corporation and TESSERA
 * TECHNOLOGY INC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//=============================================================================
// CWaitingTimeCalculator NX
//=============================================================================
#include "stdwin.h"
#include "WaitingTimeCalculator.h"


//-----------------------------------------------------------------------------
// @outline		RXgN^
// @explanation	RXgN^
// @param		none
// @return		none
//-----------------------------------------------------------------------------
CWaitingTimeCalculator::CWaitingTimeCalculator(void)
    : m_bFrequency(32), m_fFullSpeed(TRUE)
{
}


//-----------------------------------------------------------------------------
// @outline		fXgN^
// @explanation	fXgN^
// @param		none
// @return		none
//-----------------------------------------------------------------------------
CWaitingTimeCalculator::~CWaitingTimeCalculator(void)
{
}


//-----------------------------------------------------------------------------
// @outline		s܂B
// @explanation	s܂B
// @param		[IN]bFrequency  gw肵܂B(:)32MH  0x20)
//              [IN]fFullSpeed  tXs[hE[h̏ꍇɂ TRUE  ChE{e[W̏ꍇɂ FALSE w肵܂B
// @return		none
//-----------------------------------------------------------------------------
void CWaitingTimeCalculator::Initialize(BYTE bFrequency, BOOL fFullSpeed)
{
    m_bFrequency = bFrequency;
    m_fFullSpeed = fFullSpeed;
}


//-----------------------------------------------------------------------------
// @outline		Reset(tCS1)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	Reset(tCS1)  ő҂Ԃ擾܂B(PʁFmsec)
// @param		none
// @return		Reset(tCS1)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetResetTcs1() 
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;
    
    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB
    
    dBuff = ((DOUBLE)255 / m_bFrequency) / MILI_SECOND;
    
    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		BlockErase(tCS3)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	BlockErase(tCS3)  ő҂Ԃ擾܂B(PʁFmsec)
// @param		[IN]fCodeFlash  R[hEtbV̏ꍇ TRUE Af[^EtbV̏ꍇ FALSE w肵܂B
// @return		BlockErase(tCS3)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetBlockEraseTcs3(BOOL fCodeFlash)
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    if (TRUE == m_fFullSpeed) {
        if (TRUE == fCodeFlash) {
            dBuff = ((DOUBLE)67731 / m_bFrequency + 255098) / MILI_SECOND;
        }
        else {
            dBuff = ((DOUBLE)281423 / m_bFrequency + 264790) / MILI_SECOND;
        }
    }
    else {
        if (TRUE == fCodeFlash) {
            dBuff = ((DOUBLE)59455 / m_bFrequency + 265331) / MILI_SECOND;
        }
        else {
            dBuff = ((DOUBLE)248862 / m_bFrequency + 299307) / MILI_SECOND;
        }
    }

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    return dwMiliSecond;    
}


//-----------------------------------------------------------------------------
// @outline		BlockBlankCheck(tCS4)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	BlockBlankCheck(tCS4)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       [IN]dwStartAddress  JnAhXw肵܂B
//              [IN]dwEndAddress    IAhXw肵܂B
//              [IN]dwBlockCount    ubNw肵܂B
//              [IN]fCodeFlash      R[hEtbV̏ꍇ TRUE Af[^EtbV̏ꍇ FALSE w肵܂B
// @return		BlockErase(tCS3)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetBlockBlankCheckTcs4(DWORD dwStartAddress, DWORD dwEndAddress, DWORD dwBlockCount, BOOL fCodeFlash)
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    if (TRUE == m_fFullSpeed) {
        if (TRUE == fCodeFlash) {
            dBuff = (((DOUBLE) 3805 / m_bFrequency) + 91 + 
                    (((DOUBLE) 1457 / m_bFrequency) + 80) * dwBlockCount +
                    (((DOUBLE)  203 / m_bFrequency) + 18) * getFlashAccessFrequency(dwStartAddress, dwEndAddress)) / MILI_SECOND;
        }
        else {
            dBuff = (((DOUBLE) 2503 / m_bFrequency) + 86 +
                    (((DOUBLE) 5827 / m_bFrequency) + 318) * dwBlockCount) / MILI_SECOND;
        } 
    }
    else {
        if (TRUE == fCodeFlash) {
            dBuff = (((DOUBLE) 3799 / m_bFrequency) + 134 + 
                    (((DOUBLE) 1259 / m_bFrequency) + 278) * dwBlockCount +
                    (((DOUBLE)  199 / m_bFrequency) + 57) * getFlashAccessFrequency(dwStartAddress, dwEndAddress)) / MILI_SECOND;
        }
        else {
            dBuff = (((DOUBLE) 2494 / m_bFrequency) + 168 +
                    (((DOUBLE) 5035 / m_bFrequency) + 1110) * dwBlockCount) / MILI_SECOND;
        } 
    }

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;    
}


//-----------------------------------------------------------------------------
// @outline		Programming(tCS5)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	Programming(tCS5)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       [IN]fCodeFlash  R[hEtbV̏ꍇ TRUE Af[^EtbV̏ꍇ FALSE w肵܂B
// @return		Programming(tCS5)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetProgrammingTcs5(BOOL fCodeFlash)
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    if (TRUE == fCodeFlash) {
        dBuff = ((DOUBLE)1432 / m_bFrequency) / MILI_SECOND;
    }
    else {
        dBuff = ((DOUBLE)346 / m_bFrequency) / MILI_SECOND;
    }

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;    
}


//-----------------------------------------------------------------------------
// @outline		Programming(tDS5)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	Programming(tDS5)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       [IN]fCodeFlash  R[hEtbV̏ꍇ TRUE Af[^EtbV̏ꍇ FALSE w肵܂B
// @return		Programming(tDS5)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetProgrammingTds5(BOOL fCodeFlash)
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    if (TRUE == m_fFullSpeed) {
        if (TRUE == fCodeFlash) {
            dBuff = (((DOUBLE)113502 / m_bFrequency) + 71753) / MILI_SECOND;
        }
        else {
            dBuff = (((DOUBLE)309870 / m_bFrequency) + 219761) / MILI_SECOND;
        }
    }
    else {
        if (TRUE == fCodeFlash) {
            dBuff = (((DOUBLE)107803 / m_bFrequency) + 138891) / MILI_SECOND;
        }
        else {
            dBuff = (((DOUBLE)287076 / m_bFrequency) + 488315) / MILI_SECOND;
        }
    }

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // }[W
    dwMiliSecond += MARGIN_WAIT;

    return dwMiliSecond;    
}


//-----------------------------------------------------------------------------
// @outline		Programming(tSS5)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	Programming(tSS5)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       [IN]dwStartAddress  JnAhXw肵܂B
//              [IN]dwEndAddress    IAhXw肵܂B
//              [IN]dwBlockCount    ubNw肵܂B
//              [IN]fCodeFlash      R[hEtbV̏ꍇ TRUE Af[^EtbV̏ꍇ FALSE w肵܂B
// @return		Programming(tSS5)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetProgrammingTss5(DWORD dwStartAddress, DWORD dwEndAddress, DWORD dwBlockCount, BOOL fCodeFlash)
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    if (TRUE == m_fFullSpeed) {
        if (TRUE == fCodeFlash) {
            dBuff = (((DOUBLE) 1732 / m_bFrequency) + 36 +
                    (((DOUBLE) 7096 / m_bFrequency) + 892) * dwBlockCount + 
                    (((DOUBLE)  182 / m_bFrequency) +  17) * getFlashAccessFrequency(dwStartAddress, dwEndAddress)) / MILI_SECOND;
        }
        else {
            dBuff = (((DOUBLE)  397 / m_bFrequency) + 30 +
                    (((DOUBLE)28382 / m_bFrequency) + 3568) * dwBlockCount) / MILI_SECOND;
        }
    }
    else {
        if (TRUE == fCodeFlash) {
            dBuff = (((DOUBLE) 1732 / m_bFrequency) + 36 +
                    (((DOUBLE) 4351 / m_bFrequency) + 7324) * dwBlockCount + 
                    (((DOUBLE)  184 / m_bFrequency) + 44) * getFlashAccessFrequency(dwStartAddress, dwEndAddress)) / MILI_SECOND;
        }
        else {
            dBuff = (((DOUBLE)  398 / m_bFrequency) + 58 +
                    (((DOUBLE)17403 / m_bFrequency) + 29293) * dwBlockCount) / MILI_SECOND;
        }
    }

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // }[W
    dwMiliSecond += MARGIN_WAIT;

    return dwMiliSecond;    
}


//-----------------------------------------------------------------------------
// @outline		BaudRateSet(tCS6)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	BaudRateSet(tCS6)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       none
// @return		BaudRateSet(tCS6)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetBaudRateSetTcs6()
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    dBuff = (DOUBLE)4735 / MILI_SECOND;

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;
    
    // }[W
    dwMiliSecond += MARGIN_WAIT_TCS6;

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		SecuritySet(tCS7)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	SecuritySet(tCS7)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       none
// @return		SecuritySet(tCS7)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetSecuritySetTcs7()
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    dBuff = ((DOUBLE)168 / m_bFrequency) / MILI_SECOND;

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		SecuritySet(tDS7)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	SecuritySet(tDS7)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       none
// @return		SecuritySet(tDS7)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetSecuritySetTds7()
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    if (TRUE == m_fFullSpeed) {
        dBuff = (((DOUBLE)277095 / m_bFrequency) + 1027564) / MILI_SECOND;
    }
    else {
        dBuff = (((DOUBLE)242909 / m_bFrequency) + 1075967) / MILI_SECOND;
    }

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		SecurityGet(tCS8)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	SecurityGet(tCS8)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       none
// @return		SecurityGet(tCS8)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetSecurityGetTcs8()
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    dBuff = ((DOUBLE)154 / m_bFrequency) / MILI_SECOND;

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // }[W
    dwMiliSecond += MARGIN_WAIT;

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		SecurityGet(tSD8)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	SecurityGet(tSD8)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       none
// @return		SecurityGet(tSD8)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetSecurityGetTsd8()
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    dBuff = ((DOUBLE)212 / m_bFrequency) / MILI_SECOND;

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // }[W
    dwMiliSecond += MARGIN_WAIT;

    return dwMiliSecond;
}

//-----------------------------------------------------------------------------
// @outline		SecurityRelease(tCS9)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	SecurityRelease(tCS9)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       [IN]dwCodeFlashBlockCount   R[hEtbV̑ubNw肵܂B
//              [IN]dwDataFlashBlockCount   f[^EtbV̑ubNw肵܂Bif[^EtbV񓋍ڕȉꍇ 0 w肵ĂBj
// @return		SecurityRelease(tCS9)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetSecurityReleaseTcs9(DWORD dwCodeFlashBlockCount, DWORD dwDataFlashBlockCount)
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    if (TRUE == m_fFullSpeed) {
        if (0UL <dwDataFlashBlockCount) {
            dBuff = (((DOUBLE) 146110 / m_bFrequency) + 511868 + 
                    (((DOUBLE)   1457 / m_bFrequency) + 80 ) * dwCodeFlashBlockCount +
                    (((DOUBLE)   5827 / m_bFrequency) + 318) * dwDataFlashBlockCount +
                    (((DOUBLE)    203 / m_bFrequency) + 18 ) * getFlashAccessFrequency(dwCodeFlashBlockCount)) / MILI_SECOND;
        }
        else {
            dBuff = (((DOUBLE) 145783 / m_bFrequency) + 511837 + 
                    (((DOUBLE)   1457 / m_bFrequency) + 80 ) * dwCodeFlashBlockCount +
                    (((DOUBLE)    203 / m_bFrequency) + 18 ) * getFlashAccessFrequency(dwCodeFlashBlockCount)) / MILI_SECOND;
        }
    }
    else {
        if (0UL <dwDataFlashBlockCount) {
            dBuff = (((DOUBLE) 128408 / m_bFrequency) + 534723 + 
                    (((DOUBLE)   1259 / m_bFrequency) + 278 ) * dwCodeFlashBlockCount +
                    (((DOUBLE)   5035 / m_bFrequency) + 1110) * dwDataFlashBlockCount +
                    (((DOUBLE)    199 / m_bFrequency) + 57 ) * getFlashAccessFrequency(dwCodeFlashBlockCount)) / MILI_SECOND;
        }
        else {
            dBuff = (((DOUBLE) 128084 / m_bFrequency) + 534653 + 
                    (((DOUBLE)   1259 / m_bFrequency) + 278 ) * dwCodeFlashBlockCount +
                    (((DOUBLE)    199 / m_bFrequency) + 57 ) * getFlashAccessFrequency(dwCodeFlashBlockCount)) / MILI_SECOND;
        }
    }

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		CheckSum(tCS10)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	CheckSum(tCS10)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       [IN]fCodeFlash  R[hEtbV̏ꍇ TRUE Af[^EtbV̏ꍇ FALSE w肵܂B
// @return		CheckSum(tCS10)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetCheckSumTcs10(BOOL fCodeFlash)
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    if (TRUE == fCodeFlash) {
        dBuff = ((DOUBLE)203 / m_bFrequency) / MILI_SECOND;
    }
    else {
        dBuff = ((DOUBLE)219 / m_bFrequency) / MILI_SECOND;
    }

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		CheckSum(tSD10)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	CheckSum(tSD10)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       [IN]R[hEtbV̏ꍇ TRUE Af[^EtbV̏ꍇ FALSE w肵܂B
// @return		CheckSum(tSD10)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetCheckSumTsd10(DWORD dwBlockCount)
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    dBuff = (((DOUBLE)72 / m_bFrequency) +
            ((DOUBLE)30720 / m_bFrequency) * dwBlockCount) / MILI_SECOND;

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;
}

//-----------------------------------------------------------------------------
// @outline		SilliconSignature(tCS11)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	SilliconSignature(tCS11)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       none
// @return		SilliconSignature(tCS11)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetSiliconSignatureTcs11()
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    dBuff = ((DOUBLE)111 / m_bFrequency) / MILI_SECOND;

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		SilliconSignature(tSD11)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	SilliconSignature(tSD11)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       none
// @return		SilliconSignature(tSD11)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetSiliconSignatureTsd11()
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    dBuff = ((DOUBLE)512 / m_bFrequency) / MILI_SECOND;

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		Af[^M̃f[^`f[^(tDT11)  ő҂Ԃ擾܂B(PʁFmsec)
// @explanation	Af[^M̃f[^`f[^(tDT11)  ő҂Ԃ擾܂B(PʁFmsec)
// @param       none
// @return		Af[^M̃f[^`f[^(tDT11)  ő҂ԂԂ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::GetTdt()
{
    DWORD   dwMiliSecond = 0UL;
    DOUBLE  dBuff        = 0;

    // MEMO:tXs[hE[hуChE{e[W[hƂɓvZB

    dBuff = ((DOUBLE)10 / m_bFrequency) / MILI_SECOND;

    // _ȉ͐؂グB
    dwMiliSecond = (DWORD)dBuff;
    dwMiliSecond = (dBuff - (DOUBLE)dwMiliSecond) ? dwMiliSecond + 1 : dwMiliSecond;

    // MEMO:}[W
    if (MIN_WAIT > dwMiliSecond) {
        dwMiliSecond = MIN_WAIT;
    }

    return dwMiliSecond;
}


//-----------------------------------------------------------------------------
// @outline		FlashANZX񐔂擾܂B
// @explanation	FlashANZX񐔂擾܂B
// @param		[IN]dwStartAddress  JnAhXw肵܂B
//              [IN]dwEndAddress    IAhXw肵܂B
// @return		FlashANZX񐔂Ԃ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::getFlashAccessFrequency(DWORD dwStartAddress, DWORD dwEndAddress)
{
    DWORD   dwAccessFrequency = 0UL;

    // MEMO:ʓ̊Z͏_ȉ͐؂̂
    dwAccessFrequency  = ((DWORD)(dwEndAddress / 0x40000)) - ((DWORD)(dwStartAddress / 0x40000)) + 1;

    return dwAccessFrequency;
}


//-----------------------------------------------------------------------------
// @outline		FlashANZX񐔂擾܂B
// @explanation	FlashANZX񐔂擾܂B
// @param		[IN]dwCodeFlashBlockCount  R[hEtbV̑ubNw肵܂B
// @return		FlashANZX񐔂Ԃ܂B
//-----------------------------------------------------------------------------
DWORD CWaitingTimeCalculator::getFlashAccessFrequency(DWORD dwCodeFlashBlockCount)
{
    DWORD   dwAccessFrequency = 0UL;
    DOUBLE  dBuff             = 0;


    dBuff = (DOUBLE)dwCodeFlashBlockCount / 256;

    // _ȉ͐؂グB
    dwAccessFrequency = (DWORD)dBuff;
    dwAccessFrequency = (dBuff - (DOUBLE)dwAccessFrequency) ? dwAccessFrequency + 1 : dwAccessFrequency;

    return dwAccessFrequency;
}
